// RELATED SOFTWARE: Scilab >= 5.5.0, Scilab 6.0 
// PACKAGE  : uman : Unified user MANual for Scilab, external modules and resources
// VERSION  : 2.1 - 2016-10-30
// AUTHOR   : Samuel GOUGEON - private individual
// COPYRIGHT: Samuel GOUGEON -  2013 - 2016
// LICENSE for uman : CeCILL : http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt 
//
// DISCLAIMER: This toolbox is provided as is, with NO WARRANTY of any kind.
//   Use it to your own risks & responsability.

function uman(varargin)
    tmp = getversion("scilab");  
    scilab6 = tmp(1)>5;  
    // if scilab6, oldEmptyBehaviour("on"), end
    RelCharWidth = 1 // init Scaling factor for non latin characters width (ja,zh)
    noWebBrowserMsg = _("uman","uman: don''t know how to open your web browser\n")

    // FORMATING and partially PARSING OPTIONS
    // ---------------------------------------
    if argn(2)==0 then
        uman("uman","e")
        return
    end
    if argn(2)==3 then
        tmp = convstr(varargin(3))
        if grep(tmp,"l")~=[]
            varargin(1) = varargin(1)+tmp
        else
            varargin(1) = "-"+tmp+part(varargin(1), 2:$)
        end
    end
    if argn(2)==1 then
        if varargin(1)=="-"  then
            fun0 = "-"
            options = ""
        elseif part(varargin(1),1)=="-"
            if grep(varargin(1),"c")~=[] then
                clc
            end
            uman("uman")
            return
        else
            fun0 = varargin(1)
            options = ""
        end
    else
        if varargin(1)=="-" | part(varargin(1),1)~="-" then
            fun0 = varargin(1)
            options = varargin(2)
        else
            fun0 = varargin(2)
            options = varargin(1)
        end
    end

    options = convstr(options)
    options0 = options  // useful in case of redirection (recursive => call)
    [options, LANG] = uman_splitOptionsLang(options)
    // Option "@" is canceled when it is not alone:
    if grep(options,"@")~=[] & length(strsubst(options,"@",""))>0 
        options = strsubst(options, "@", "")
    end

    // ========================================================================

    // MAILING LISTS
    // ------------- 
    if grep(options, "@")~=[] then
        // Building the query
        MailingListsQuery = "http://mailinglists.scilab.org/template/NamlServlet.jtp?macro=search_page&node=2602239&query="
        // TESTS in Subjects:
        //fun = "test & scilab"; r0 = "subject%3Atest+subject%3Ascilab"
        //r = test(fun), r==r0
        //fun = "test | scilab"; r0 = "subject%3A(test+OR+scilab)"
        //r = test(fun), r==r0
        //fun = "{test | scilab}"; r0 = "subject%3A(test+OR+scilab)"
        //r = test(fun), r==r0
        //fun = "~{test|scilab}"; r0 = "NOT+subject%3A(test+OR+scilab)"
        //r = test(fun), r==r0
        //fun = "test & scilab & ~{java|cpp}";  
        //r0 = "subject%3Atest+subject%3Ascilab+NOT+subject%3A(java+OR+cpp)";
        //r = test(fun), r==r0
        //fun = "~{java|cpp} & test & scilab";  
        //r0 = "NOT+subject%3A(java+OR+cpp)+subject%3Atest+subject%3Ascilab";
        //r = test(fun), r==r0
        //fun = "test & scilab & {image|signal} & ~{java|cpp}"
        //r0 = "subject%3Atest+subject%3Ascilab+subject%3A(image+OR+signal)+NOT+subject%3A(java+OR+cpp)";
        //r = test(fun), r==r0
        
        // In subjects or in full messages?
        s = "+"
        fun = fun0
        if part(fun,$)=="!"  // search only in subjects of messages.
            s = "+subject%3A"
            fun = part(fun,1:$-1)
        else                   // search in subjects + bodies
        end
        d = "!" // regexp delimiter

        // Authors part:
        [s0, s1, tmp, txt] = regexp(fun, "/^([a-z0-9_\-\.\s]+)\>.*/i","o")
        txt = stripblanks(txt)
        if txt ~= ""
            authors = strcat("author:"+stripblanks(tokens(txt,"|"))+" ")
            fun = " " + part(fun, s1+1:$)
        else
            authors = ""
        end

        // Max age part:
        tmp = tokens(fun, "<")
        if size(tmp,1)>1
            max_age = "&days="+tmp($)
            fun = strcat(tmp(1:$-1),"<")
        else
            max_age = ""
        end
        fun = stripblanks(fun)
        // Topics part
        if fun~=""
            if grep(fun,"&")==[] & sum(grep(fun, ["~" "(" ")"]))<2
                fun = "("+fun+")"
            end
            tmp = ["%" "!" "?" "#" "[" "]" "-" "+" "*" "/" "^" "$" ]
            tmp = uman_strsubst(fun, tmp, "%"+dec2hex(ascii(tmp)))
            // ":" must be protected with "\:" because of "subject:","author:","message:"
            tmp = uman_strsubst(tmp, ":", "%5C%3A") 
            //
            tmp = uman_strsubst(tmp, [ "(" ")" ],["{" "}"])
            tmp = uman_strsubst(tmp, d+["\s*\&\s*\~\{(.+)\}" ..    // & ~{..}
                                        "\&?\s*\~\s*" ]+d,..    // & ~ | ~
                                       ["+NOT"+s+"{$1}" "+NOT"+s ], "r")
            tmp = uman_strsubst(tmp, d+["\s*\|\s*"]+d, ["+OR+"], "r")
            tmp = uman_strsubst(tmp, d+["\s*&\s*" "\{\s*?(.+?)\s*\}"]+d, [s "($1)"], "r")
            if grep(tmp,d+"^\"+["+NOT\+" s]+d,"r")==[]
                tmp = s+tmp
            end
        else
            tmp = ""
        end
        tmp = authors + part(tmp, 2:$) + max_age
        MailingListsQuery = MailingListsQuery + tmp
        // DISPLAY
        uman_openURL(MailingListsQuery, noWebBrowserMsg)
        // QUIT
        return
    end
    // ========================================================================
    //
    // PREPROCESSING "wb" pattern
    // --------------------------
    // The pattern may have from 1 to 4 parts a,b > c < d :
    // $component1|component2:author1|author2>topic1|topic2..<Ndays
    components = []
    authors = []
    urlQuery = ""
    tmp = strsplit(fun0,">")
    if size(tmp,1)>1
        trail = strcat(tmp(2:$), ">")
        tmp = stripblanks(strsplit(tmp(1), "/"))
        if size(tmp,1)==1 & tmp~="" then
            authors = stripblanks(tokens(tmp,"|"))
        else
            components = stripblanks(tokens(tmp(1),"|"))
            if size(components,1)==1 & components==""
                components = []
            end
            authors = stripblanks(tokens(tmp(2),"|"))
            if size(authors,1)==1 & authors==""
                authors = []
            end
        end
        // substitutions:
        components = strcat(components,",")
        if components~=[]
            compaliases = "atomssys:atoms,atomsgui:ATOMS GUI,"+..
            "atomsportal:Atoms Portal&product=Scilab websites"+..
            "&product=Atoms Portal&component=Miscellaneous,"+..
            "atoms:atoms&component=atoms gui"+..
            "&product=Scilab websites&component=Atoms Portal"+..
            "&product=ATOMS Portal&component=Miscellaneous,"+..
            "api:api_scilab,call:call_scilab,"+..
            "codeimprove:Code improvement,cli:Console as CLI,"+..
            "iodata:Data import/export,demos:Demonstrations,ide:desktop,"+..
            "doc:Documentation pages,"+..
            "help:Documentation pages,helpsys:Documentation system,"+..
            "link:Dynamic Link,editvar:Editvar/Browsevar,"+..
            "browsevar:Editvar/Browsevar,varbrowser:Editvar/Browsevar,"+..
            "filebrowser:File browser,graphicsexport:Graphics export,"+..
            "io:Input / output,instal:Install,java:Java features,"+..
            "translation:Localization,matlab2scilab:m2sci,matfiles:Mat files,"+..
            "maths:Numerical Math,math:Numerical Math,scipad:Old editor (Scipad),"+..
            "optim:Optimization,"+..
            "parser:Parser/Lexer,parse:Parser/Lexer,lexer:Parser/Lexer,"+..
            "parallel:PVM,editor:SciNotes,stats:Statistics,text:Strings,"+..
            "tests:Tests &amp; Testing system,"+..
            "uicomp:uicontrol"
            compaliases = matrix(tokens(compaliases,["," ":"]),2,-1)
            components = uman_strsubst(","+components+",",","+compaliases(1,:)+",",..
                         ","+compaliases(2,:)+",")
            components = tokens(components,",")
        end
    else
        trail = tmp
    end
    // Components
    if components~=[] then
        urlQuery = urlQuery + strcat("component="+components+"&")
    end
    // Authors
    i = 1
    for tmp = authors'
        tmp2 = msprintf("f%d=commenter&o%d=substring&v%d=%s&",i,i,i,tmp)
        tmp3 = msprintf("f%d=reporter&o%d=substring&v%d=%s&",i+1,i+1,i+1,tmp)
        urlQuery = urlQuery + tmp2 + tmp3
        i = i+2
    end
    if authors~=[]
        urlQuery = urlQuery + "j_top=OR&"
    end
    // Topic & max_age
    tmp = strsplit(trail, "<")
    if size(tmp,1)==1
        fun0 = stripblanks(trail)
    else
        if isnum(tmp($))
            fun0 = strcat(tmp(1:$-1),"<")
            urlQuery = urlQuery + msprintf("chfieldfrom=%dd&", evstr(tmp($))) + ..
                        "chfieldto=Now&"
        else
            fun0 = stripblanks(trail)
        end
    end

    // ========================================================================

    // MERGING DEFAULT OPTIONS
    // -----------------------
    if isdef("umanDefaultOptions", "n")
        dopt = convstr(umanDefaultOptions)
        dopt = uman_strsubst(dopt,["w" "@"],"") // "w"|"@" not supported in defaults
        [dopt, dLANG] = uman_splitOptionsLang(dopt)
        // Calling options d,e,h,b,a overrides any a|s default ones
        if grep(options,["d" "e" "h" "b" "a" "u"])~=[] & grep(dopt,["a" "s"])~=[]
            dopt = uman_strsubst(dopt, ["a" "s"], "")
        end
        // Calling options d,e,h,b,a,s overrides default u
        if grep(options,["d" "e" "h" "b" "a" "s"])~=[] & grep(dopt,"u")~=[]
            dopt = uman_strsubst(dopt, "u", "")
        end
        // Calling option u overrides any d,e,h,b,a default ones
        if grep(options,"u")~=[] & grep(dopt,["d" "e" "h" "b" "a"])~=[]
            dopt = uman_strsubst(dopt, ["d" "e" "h" "b" "a"], "")
        end
        options = options+dopt
    end
    
    // FINAL LANGUAGE
    // --------------
    if LANG==""
        if ~isdef("dLANG","l") | dLANG=="" 
            LANG = part(getlanguage(), 1:2)
        else
            LANG = dLANG
        end
    end
    
    // OPTIONS ANALYSIS
    // ----------------
    fun = fun0
    if grep(options, "a")~=[] then
        options = strsubst(options,"a","deh")
    end
    fun = uman_strsubst(fun,"/^%/" ,"percent","r") 
    // This substitution will have to be cancelled in "wb" mode before calling
    //  the bug page. Nevertheless, it is required to search the right page
    //  to determine the related module (or scilab) and the related bug page 
    //  to be opened.
    
    // option "s" (summary) kills option "b" (bug), "w" (web) and "@" (mailinglists)
    if grep(options, "s")~=[] then
        options = uman_strsubst(options, ["b" "w" "@"], "")
    end
    // option "u" (usage) kills options d,e,b,h,a,s,g
    if grep(options, "u")~=[] then
        options = uman_strsubst(options, ["d" "e" "b" "h" "a" "s" "g"], "")
    end
    bugOnWeb = grep(options, "w")~=[] & grep(options,"b")~=[]

    // SETTING THE LANGUAGE
    // --------------------
    supportedLANG = [ "en_US" "fr_FR" "ja_JP" "de_DE" "pt_BR" "ru_RU" "zh_CN" "fa_IR"]
    i = grep(supportedLANG, "/^"+LANG+"_/", "r")
    if i==[] then
        LANG = "en_US"
    else
        LANG = supportedLANG(i)
    end

    // REDIRECTIONS for Symbols. Translation for "%" in fun names
    // ----------------------------------------------------------
    if ~bugOnWeb then
        fun = uman_strsubst(fun,["/\{/" "/\}/"],["[" "]"],"r") 
        sub = "assignation,great,  > ,  < ,   [    ,   ]    ,  :  ,   //   ,"
        ids = "insertion  ,less ,less,less,brackets,brackets,colon,comments,"
        
        sub = sub+"    <>    ,   ~=     ,   <=     ,   >=     ,"
        ids = ids+"comparison,comparison,comparison,comparison,"

        sub = sub+"  =  ,    ==    ,"
        ids = ids+"equal,comparison,"
        // bug with these ones, due to Scilab parser. 
        // Must be set within quotes: uman "=="
        
        sub = sub+"  &   ,  |  , ~ ,   $  , . , []  ,"
        ids = ids+"and_op,or_op,not,dollar,dot,empty,"
            // % automaticaly replaced with percent 
        
        sub = sub+"  -  ,  + ,  * , .* ,  /  , ./  ,  /. , .^  , **  , ^ , .*.,    \    ,"
        ids = ids+"minus,plus,star,star,slash,slash,slash,power,power,hat,kron,backslash,"
        
        sub = sub+"    \.     ,     *.    ,    ./.    ,    .\.    ,"
        ids = ids+"overloading,overloading,overloading,overloading,"
        
        sub = sub+"  '' ,    ;    "
        ids = ids+"quote,semicolon"
        
        sub = sub+""
        ids = ids+""
        sub = [ stripblanks(strsplit(sub, ",")) ; "," ]
        ids = [ stripblanks(strsplit(ids, ",")) ; "comma" ]
        i = find(fun==sub)
        if i~=[] then
            fun = ids(i(1))
        end
        if grep(fun, " ") then
            fun = strsubst(fun, " ", "_")
        end
    end

    // OTHER INITIALISATIONS
    // ---------------------
    // Case sensitivity for the extraction
    if convstr(fun)==fun then
        casse = "-C"    // case-insensitive
    else
        casse = ""      // case-sensitive
    end
    // Setting the unzip application vs OS to extract pages from JAR archives
    if getos() == "Windows" then
        unzip  = pathconvert(SCI+"\tools\zip\") + "unzip.exe "+casse+" -o -q " 
        //unzip2 = pathconvert(SCI+"\tools\gzip\") + "gzip.exe "
    else
        unzip = "unzip "+casse+" "
    end

    // Miscellaneous init
    fs = filesep()
    unzippedFile = ""        // = flag for "File Not Found"
    
    // Directories storing unzipped HTML files
    // ---------------------------------------
    //  Root directory
    manDir = pathconvert(SCIHOME + "\uman\")
    if fileinfo(manDir)==[] then  
        mkdir(manDir)
    end
    // Main directory for Scilab HTML extracted pages
    manScilabDir = pathconvert(manDir + "HTML_pages_Scilab\")   
    if fileinfo(manScilabDir)==[] then  
        mkdir(manScilabDir) // only once. Scilab files don't need to be refreshed
    end
    // Main directory for ATOMS and other contribs HTML extracted pages
    manAtomsDir = pathconvert(manDir + "HTML_pages_Contrib\") 
    if fileinfo(manAtomsDir)==[] then  
        mkdir(manAtomsDir)
    else
        // Refresh option => deleting all extracted HTML pages in manAtomsDir
        if grep(options, "r")~=[] then
            rmdir(manAtomsDir,"s")
            mkdir(manAtomsDir)
        end
    end
        
    // Listing JAR files: Scilab + ATOMS/Contrib(all_users) + ATOMS/Contrib(user)
    // ==========================================================================
    // We need this part also for bugOnWeb: if the pattern matches an external
    //  function (e.g. from ATOMS), the external module must be identified
    //  in order to target the right online bug report place
    jarListFile = manDir+"jarListe.txt"
    i = fileinfo(jarListFile)
    // If the list of JAR files does not exist, or if the "r" (reload / refresh) 
    //  option is used, we rebuild and record it and re-extract jhelpmap.jhm files :
    if i==[] | grep(options,"r")~=[] then
        // Building the list of JAR files
        // ------------------------------
        // Returns de column of paths to ALL JAR files (not only for LANG or en_US)
        function flist = dirjar(path)
            // path must be ended with filesep()
            if isdir(path) then
                if getos()=="Windows" then
                    flist = unix_g("dir /s /b "+path+"scilab_*_help.jar 2> nul")
                elseif getos()=="Linux"
                    flist = unix_g("find "+path+" -name scilab_*_help.jar 2> /dev/null")
                else
                    flist = unix_g("find 2>/dev/null "+path+ "scilab_*_help.jar")
                end
                if flist=="" then
                    flist = []
                end
            else
                flist = []
            end
        endfunction
        // Let's GO:
        jarList = []
        // Scilab trunk:
        path = pathconvert(pathconvert(SCI)+"modules\helptools\jar\")
        jarList = [jarList ; dirjar(path)]
        // Contrib (ATOMS and others) : ALL USERS
        path = pathconvert(pathconvert(SCI)+"contrib\")
        jarList = [jarList ; dirjar(path)]
        // USER : ATOMS
        path = pathconvert(pathconvert(SCIHOME)+"atoms\")
        jarList = [jarList ; dirjar(path)]
        // USER : other contribs
        path = pathconvert(pathconvert(SCIHOME)+"contrib\")
        jarList = [jarList ; dirjar(path)]
        // OTHER PATHS provided in umanUserPaths
        global umanUserPaths
        if umanUserPaths~=[] & type(umanUserPaths)==10
            for p = umanUserPaths(:)'
                if isdir(p)
                    jarList = [jarList ; dirjar(pathconvert(p))]
                end
            end
        end
        
        // Recording the list:
        csvWrite(jarList, jarListFile, ",")
    else
        jarList = mgetl(jarListFile)
    end
    
    // Various initializations
    // -----------------------
    module = ""
    clang = getlanguage()

    // Extraction of jhelpmap.jhm files
    // ================================
    // as well needed by the "wb" option (for forge's tickets)

    // Directory storing jhelpmap.jhm files renamed as moduleName_la_RG.jhm
    //  or scilab_la_RG.jhm, where la_RG is the language_ReGion code (en_US, etc)
    jhmListFile = manDir + "jhmListe.txt"
    mapDir = manDir + "jhelpMaps" + fs          
    if ~isdir(mapDir) then
        mkdir(mapDir)
        mkdir(mapDir+"scilab"+fs) // subdir for fixed scilab_la_RG.jhm files
    else
        if grep(options, "r")~=[] then
            mdelete(mapDir+"*.jhm")
        end
    end
    
    if i==[] | grep(options,"r")~=[] then
        // From the list of selected JAR files, for each file, extracts its 
        //  jhelpmap.jhm in a common directory, and renames it according to the
        //  module AND language tag.
        // "unzip" includes the options -C: case-insensitive -o: overwrite -q: quiet
        fs = filesep()
        pattern = pathconvert("\modules\helptools\jar\scilab_", %f)
        jhmListe = []
        for jar = jarList'
            // Module name related to the .jar file = ?
            if grep(jar, pattern)~=[]
                module = "scilab"
            else
                // The name of the module (such as stored in %helps(:,2))
                //  is retrieved through the file name ~\etc\modulename.start
                tmp = fullpath(dirname(jar)+fs+"..\etc\*.start")
                tmp = dir(tmp)
                startfile = tmp.name(1)
                module = basename(startfile)
            end
            lang = uman_strsubst(jar, "|.*?_([a-z]{2}_[A-Z]{2})_help.jar|", "$1", "r")
            subDir = "scilab_"+lang+"_help"+fs
            zippedMap = subDir + "jhelpmap.jhm"
            if getos()=="Windows"
                jar = getshortpathname(jar)    // strips blanks from the path
            end
            //command = unzip + """"+jar + """ """ + zippedMap + """ -d """+manDir+""""
            command = unzip + jar + " " + zippedMap + " -d "
            if module=="scilab"
                command = command + manScilabDir
                unzippedMapFrom = manScilabDir + subDir + "jhelpmap.jhm"
                unzippedMapTo = "scilab"+fs+module+"_"+lang+".jhm" 
            else
                command = command + manAtomsDir
                unzippedMapFrom = manAtomsDir + subDir + "jhelpmap.jhm"
                unzippedMapTo = module+"_"+lang+".jhm" 
            end
            status = host(command)
            [s, m] = movefile(unzippedMapFrom, mapDir+unzippedMapTo)
            jhmListe = [jhmListe ; [module unzippedMapTo jar ]]
        end
        // Removing duplicates .jhm: the last is kept (has overwritten previous ones)
        jhmListe = jhmListe($:-1:1,:)
        [tmp, k] = unique(jhmListe(:,2))
        jhmListe = jhmListe(gsort(k,"g","i"),:)
        csvWrite(jhmListe, jhmListFile, ",")
    else
        jhmListe = csvRead(jhmListFile,",",[],"string")
    end

    // Building 2 lists of jhmHelpMaps, ordered by lang: 
    //   1) for Scilab   2) for external modules 
    // =================================================
    function k = uman_orderJhmHMapsByLang(c, external?)
        // c = jhmListe(:,2) : column of module_##_##.jhm names
        // external? : boolean %T whether only external modules are considered
        //                     %T whether only native Scilab is considered
        // LANG and clang external contextual variables are used.
        //
        // The language has priority:
        // Chosen language (or default)
        if external? then
            // Chosen language (or default)
            k = grep(c, "|^[^\"+fs+"]+_"+LANG+"\.jhm|","r")
            // Otherwise: Session's language
            if clang~=LANG
                k = [k grep(c, "|^[^\"+fs+"]+_"+clang+"\.jhm|", "r")]
            end
            // Otherwise, finally: en_US
            if LANG~="en_US" & clang~="en_US"
                k = [k grep(c, "|^[^\"+fs+"]+_en_US\.jhm|", "r") ]
            end
        else
            k = find(c=="scilab"+fs+"scilab_"+LANG+".jhm")
            if clang~=LANG
                k = [ k find(c=="scilab"+fs+"scilab_"+clang+".jhm") ]
            end
            if LANG~="en_US" & clang~="en_US"
                k = [ k find(c=="scilab"+fs+"scilab_en_US.jhm") ]
            end
        end
    endfunction
    // Scilab list:
    k = find(jhmListe(:,1)=="scilab")
    tmp = jhmListe(k, :)
    k2 = uman_orderJhmHMapsByLang(tmp(:,2), %f)
    jhmScilab = jhmListe(k(k2),:)
    // List for external modules (ATOMS & contribs):
    k = find(jhmListe(:,1)~="scilab")
    tmp = jhmListe(k, :)
    if tmp~=[] then
        k2 = uman_orderJhmHMapsByLang(tmp(:,2), %t)
        jhmModules = jhmListe(k(k2),:)
    else
        jhmModules = []
    end

    // SEARCHING FOR THE RIGHT HELP TARGET (module, page, local macro, web pointer)
    // ====================================
    htmlName = ""
    localMacro = ""
    webId = ""      // Id for an ATOMS, a FE, or another external ressource
    jhmLine = []
    if casse=="-C"
        grepCase = "i"
    else
        grepCase = ""
    end

    if fun~="" & grep(fun, ["|" "/" "\" "*" ":" "[" "("])==[]
        // Priorities: Scilab > external modules > local macro > web pointer
        // -----------------------------------------------------------------
        funR = ""
        if grep(options, "x")==[]
            // PRIORITIES for Soft mormal mode AND "x"==[] :
            // (Shortcuts)Scilab
            // (ex2in) Scilab
            // Modules 
            // local
            // (ex2ex) Modules
            // (ex2key) Scilab
            [fun1s, optmp] = uman_get_equiv(fun, options, "shortcuts")
            [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun1s, jhmScilab, mapDir, grepCase)
            if htmlName==""
                // (ex2in) Scilab
                [fun2s, optmp] = uman_get_equiv(fun, options, "ex2in")
                if fun2s~=fun1s
                    [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun2s, jhmScilab, mapDir, grepCase)
                    if htmlName~=""
                        funR = fun2s, options = optmp
                    end
                end
            else
                funR = fun1s, options = optmp
            end
            // Modules
            if htmlName=="" & jhmModules~=[]
                [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun, jhmModules, mapDir, grepCase)
                if htmlName ~= ""
                    funR = fun
                end
            end
            // Local
            if htmlName==""
                localMacro = uman_find_localMacro(fun)
                if localMacro~=""
                    funR = fun
                end
            end
            // (ex2ex) Modules
            if htmlName=="" & localMacro=="" & jhmModules~=[]
                [fun2m, optmp] = uman_get_equiv(fun, options, "ex2ex")
                if fun2m~=fun
                    [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun2m, jhmModules, mapDir, grepCase)
                    if htmlName~=""
                        funR = fun2m, options = optmp
                    end
                end
            end
            // RESULTS: If htmlName=="" & localMacro=="" then funR==""
            //  otherwise funR~=""

        else
            // "x": ------------------------------------------------------------
            // PRIORITIES for Soft normal mode AND "x" :
            // Modules
            // (faux-amis)(ex2in) Scilab
            // (ex2ex) Modules
            // local
            // (shortcuts) Scilab
            // (ex2key) Scilab

            // Modules
            if jhmModules~=[]
                [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun, jhmModules, mapDir, grepCase)
            end
            // (faux-amis)(ex2in) Scilab
            if htmlName==""
                [fun1s, optmp] = uman_get_equiv(fun, options, "exin2in")
                [fun1s, optmp] = uman_get_equiv(fun1s, options, "ex2in")
                [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun1s, jhmScilab, mapDir, grepCase)
                if htmlName~=""
                    funR = fun1s, options = optmp
                end
            end
            // (ex2ex) Modules 
            if htmlName=="" & jhmModules~=[] 
                [fun1m, optmp] = uman_get_equiv(fun, options, "ex2ex") 
                [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun1m, jhmModules, mapDir, grepCase) 
                if htmlName~="" 
                    funR = fun1m, options = optmp
                end
            end
            // local
            if htmlName==""
                localMacro = uman_find_localMacro(fun)
                if localMacro~=""
                    funR = fun
                end
            end
            // (shortcuts) Scilab
            if htmlName=="" & localMacro==""
                [fun2s, optmp] = uman_get_equiv(fun, options, "shortcuts") 
                if fun2s~=fun1s
                    [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun2s, jhmScilab, mapDir, grepCase) 
                    if htmlName~="" 
                        funR = fun2s, options = optmp
                    end
                end
            end
        end     // if "x"
        
        // (ex2key) Scilab (common to "x" & ~"x" at the end):
        if htmlName=="" & localMacro==""
            [fun3, optmp] = uman_get_equiv(fun, options, "ex2key")
            funR = fun3, options = optmp
        end
    fun = funR
    end

    // Module identification & informations (atoms, forge, website)
    // ------------------------------------------------------------
    module = ""
    if jhmLine~=[] then
        module = jhmLine(1)
    end
    // 
    AF = uman_get_atoms_forge()
    AFid = AF(:,2)
    webRef = []
    if module~="scilab"  then
        k = []
        if module~="" then
            k = find(AFid==module)
            if k==[]
                k = find(convstr(AFid)==convstr(module))
            end
        else
            if fun==convstr(fun)
                k = find(convstr(AFid)==fun)
            else
                k = find(AFid==fun)
            end
            if k==[]    // We then try after redirection:
                fun1e = uman_get_equiv(fun, options, "ex2ex")
                if fun1e==convstr(fun1e)
                    k = find(convstr(AFid)==fun1e)
                else
                    k = find(AFid==fun1e)
                end
            end
        end
        if k~=[]
            webRef = AF(k,:)  // 1: type  2: module_id  3: forge_id  4: website
            module = webRef(2)
        end
    end

    // List of installed ATOMS modules
    // -------------------------------
    atomsInstalled = atomsGetInstalled()
    if atomsInstalled~=[] then
        atomsInstalled(:,4) = getshortpathname(pathconvert(atomsInstalled(:,4)))    
    end
    isATOMSinstalled = (find(atomsInstalled(:,1)==module)~=[])

    // Now we have:
    // -----------
    // htmlName: the HTML reference to extract (if found)
    // module  : the name of the external module (if relevant)
    // isATOMSinstalled : boolean about the module (if any)
    // localMacro: the name of the local macro (if it exists)
     
     // For "wb", required infos are now available => DISPLAY
     
    // ========================================================================
    
    // BUGZILLA | TICKET / FORGE | WEBSITE
    // ===================================
    // Default place for Scilab or mixed items:
    //  "&o1=regexp&f1=short_desc&v1=(^|[^a-zA-Z_])£([^0-9a-zA-Z_%3D%3B]|$)"+..
    BugzillaSearch = "http://bugzilla.scilab.org/buglist.cgi" + ..
      "?product=Scilab software&query_format=advanced" + ..
      "&short_desc_type=regexp&short_desc=£"+..
      "&bug_status=NEW&bug_status=ASSIGNED&bug_status=REOPENED"+..
      "&bug_status=RESOLVED&"
      // £ will be replaced with the item name
//    if %t | urlQuery==""
        BugzillaSearch = BugzillaSearch + urlQuery + "order=resolution,bug_id DESC"
//    else
//        BugzillaSearch = BugzillaSearch + urlQuery + ..
//                        "order=changeddate DESC%2Cbug_id DESC&"
//    end
    BugReport = ""
    if bugOnWeb
        // Bug number provided: no other infos required => Display
        // -------------------------------------------------------
        if grep(fun0,"/^[0-9]+$/","r")==1
            if  evstr(fun0)>76  // item = bug number
                url = "http://bugzilla.scilab.org/"+fun0
                uman_openURL(url, noWebBrowserMsg)
                return
            end
        end
        // Setting the right bug report place for externals
        // ------------------------------------------------
        // ATOMS modules registered for "uman"
        if module~="scilab" & webRef~=[] then
            // If the external module is an ATOMS with a public forge: 
            // Tickets are on it
            if webRef(3)~=""
                BugReport = "http://forge.scilab.org/index.php/p/"+webRef(3)+"/issues/"
                BugzillaSearch = BugReport + "search/?q="
                if htmlName~=""
                    BugzillaSearch = BugzillaSearch + fun
                else
                    BugzillaSearch = BugzillaSearch + fun0
                end
                
            else 
                // If in ATOMS (without forge): => we target ATOMS comments
                if webRef(1)=="a"
                    tmp = "http://atoms.scilab.org/toolboxes/"+module+"/#comments_remove"
                elseif webRef(1)=="fe"  // FileExchange
                    tmp = stripblanks(webRef(4))+"/#comments_remove"
                    
                elseif webRef(1)=="fo"  // Forge without ATOMS
                    tmp = "http://forge.scilab.org/index.php/p/" + ..
                        stripblanks(webRef(2))+"/issues/search/?q="
                    if htmlName~=""
                        tmp = tmp + fun
                    else
                        tmp = tmp + fun0
                    end
                else // Otherwise we target verbatim the indicated website
                    tmp = stripblanks(webRef(4))
                end
                BugReport = tmp
                BugzillaSearch = tmp
            end
            
        // Installed ATOMS module BUT unregistered for uman:
        elseif isATOMSinstalled
            tmp = "http://atoms.scilab.org/toolboxes/"+module+"/#comments_remove"
            BugReport = tmp
            BugzillaSearch = tmp
        
        // Other Contrib modules
        elseif module~="scilab" & htmlName~=""
            BugzillaSearch = ""
        end

        if BugzillaSearch~=""  then
            fun = strsubst(fun, "/^percent/", "%", "r")
            //symbols = "><[]:<>~=<=>==&|$.**-+././.^.*.\.'';"
            if grep(fun,"/")~=[]
                delim = "!"
            else
                delim = "/"
            end
            [prepend, append] = ("","")
            if grep("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ",part(fun,1))~=[]
                prepend = "(^|[^a-zA-Z_])"
            end
            if fun~="" & grep("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ",part(fun,$))~=[]
                append = "([^0-9a-zA-Z_%3B]|$)"
              // ([^0-9a-zA-Z_%3D%3B]|$) => ([^0-9a-zA-Z_%3B]|$), otherwise
              //  'uman figure_size wb' does not detect  '... a.figure_size=v...'
            end
            tmp = uman_strsubst(fun, ..
                delim+["\\"   "&"   "\*"  "\."  "\-"  "\:"  "\|\|"  "\]"  "\["  "\("  "\)"  "\$"    "\^" "/" "\?" "\#"]+delim, ..
                      ["\\\\" "%26" "\\*" "\\." "\\-" "\\:" "\\|\\|" "\\]" "\\[" "\\(" "\\)" "\\%24" "\\^"  "%2F" "\\%3F" "\\%23"], "r")
            if tmp==fun
                tmp = fun0
            end
            if tmp==""
                tmp = ".*"
            end
            if fun~=""
                url = strsubst(BugzillaSearch,"£", prepend+"("+tmp+")"+append)
            else
                url = strsubst(BugzillaSearch,"£", "")
            end
            // tmp must be () in case of it has some | alternatives
    
            uman_openURL(url, noWebBrowserMsg)
        else
            // NO SITE TO SEE BUGS....
        end
        return
    end

    // ========================================================================

    // Targetting ONLINE help page (or site)
    // =====================================
    printWeb = ( (grep(options,"w")~=[] & grep(options,"b")==[]) | ..
                   ( unzippedFile=="" & grep(options0,"w")~=[] ) ..
                 )
    if printWeb
        url = ""
        if webRef~=[]
            // ATOMS module => open the ATOMS page
            if webRef(1)=="a"
                url = "https://atoms.scilab.org/toolboxes/"+webRef(2)
                
            // FileExchange => open the FE page
            elseif webRef(1)=="fe"
                url = webRef(4)
            
            // Forge with no ATOMS => open the Forge home page
            elseif webRef(1)=="fo"
                url = "http://forge.scilab.org/index.php/p/"+webRef(2)
            
            // Other external resource
            else
                url = webRef(4)
            end
        elseif or(module==["" "scilab"])
            url = "https://help.scilab.org/docs/current/"
            supported_online= ["en_US" "fr_FR" "pt_BR" "ru_RU" "ja_JP"]
            if and(LANG~=supported_online)
                langH = "en_US"
            else
                langH = LANG
            end
            // if unzippedFile~=""
            if htmlName ~= ""
                // url = url+langH+"/"+fun+".html"
                // url = url+langH+"/"+basename(unzippedFile)+".html"
                // For Type, Equal, etc: URL(web) ~= URL(jHelpMap):
                if grep(htmlName, "/.+\-[0-9a-f]{32}\.html/","r")
                    htmlName = strsubst(htmlName, "/\-[0-9a-f]{32}/","","r")
                end
                url = url + langH + "/" + htmlName
            else
                // query example: "cholesky+factorization"+AND+factors
                url = url + langH + "/search/" + fun0
            end
        end
        if url~=""
            uman_openURL(url, noWebBrowserMsg)
        else
            disp("NO WEB REFERENCE AVAILABLE")
        end
        // We don't return, because the "w" allows also "g"
    end

    // ========================================================================
    
    // Setting the flag allowing to target the console
    // ...............................................
    //  - whatever is the "g" status (can duplicate the output)
    //  - whether "w" is not used, OR (NOT "wb" AND NOT Scilab module)
    printConsole = (grep(options, ["w" "@"])==[] | (~bugOnWeb & module~="scilab" ))

    if printConsole & grep(options,"j")==[] 
        diary([], "pause") // http://bugzilla.scilab.org/show_bug.cgi?id=12551
    end
    noContent = %f  // set it to %t if [pattern] has no dedicated page

    // HTML Page's extraction, if needed and not yet in cache
    // ------------------------------------------------------
    if htmlName~="" then
       jar = jhmLine(3)  // JAR file that htmlName belongs to
       [tmp, tmp, tmp, lang] = regexp(jhmLine(2), "|_(.._..)\.jhm|")
       subDir = "scilab_"+lang+"_help"
       // is the HTML of this jar unzipped ?
       zippedFile = subDir + fs + htmlName
        if module=="scilab" 
            outDir = manScilabDir   // ended with / 
        else  // ATOMS and other contribs are mixed
            outDir = manAtomsDir    // idem
        end
        unzipped = outDir + subDir + fs + htmlName
       if isfile(unzipped) & grep(options, "r")==[]
           unzippedFile = unzipped
       else
           if isfile(unzipped) 
               deletefile(unzipped)
           end
           if jar~=[]
               // We extract the HTML file:
               command = unzip + jar + " " + zippedFile + " -d " + outDir
               status = host(command)
               if status==0 // file found and extracted
                   unzippedFile = unzipped
               end
           else
               status = 9   // zip file not found
           end
       end
    end
    // -------------------------------------------------------

    // The required help file exists...
    // --------------------------------
    if unzippedFile~=""       // we go on on processing
        r = uman_grab(unzippedFile, fun, LANG)
        r.fun = fun0

        // Targetting the console
        // ......................
        msg2 = ""
        if  printConsole
            if grep(options,"c")~=[] then
                clc()
            elseif grep(options,"u")==[]    // separating zone
                tmp = lines()
                write(%io(2), part("=", ones(1,tmp(1))))
            end
            // item's page
            // - - - - - -
            msg0 = ""
            if grep(options, "s")==[]
                if module~="scilab"
                    if webRef(1)=="a"
                        r.bug_report = BugReport
                        r.bugs_known = BugzillaSearch
                    else
                        r.bug_report = ""
                        r.bugs_known = ""
                    end
                end
                // Display of the proper help page
                uman_disp(r, options)

            // Parent summary
            // - - - -- - - -
            else 
                tmp = strsplit(unzippedFile, fs)
                zippedFile = tmp($-1) + fs + r.parentSummary + ".html"
                tmp = strsplit(unzippedFile, fs)
                outDir = strcat(tmp(1:$-2), fs)
                unzippedFile = dirname(unzippedFile) + fs + r.parentSummary + ".html"

                if fileinfo(unzippedFile)==[]
                    command = unzip + jar + " " + zippedFile + " -d " + outDir
                    status = host(command)  // status==0  <=> OK
                end
                
                // Parsing the summary page ++++++++++++++++++++++++++
                html = mgetl(unzippedFile)                
                search  = [ "<html>" 
                            "<body>" 
                            "<head>.*?</head>" 
                            "<div class=""manualnavbar"">.*?</div>" 
                            "</body>" 
                            "</html>"
                            "<br />"
                            ]
                search = "|" + search + "|"
                replace = ["<div>" "" "" "" "" "</div>" ascii(10) ]
                html2 = uman_strsubst(strcat(html, " "), search, replace, "r")
                tmp = xmlReadStr(html2)
                simpleContent = %t
                
                // If language = ja|zh: we must assess the fraction of content in
                // in japanese|chinese vs latin, in order to tune the average 
                // character width used to wrap long lines
                lang = part(LANG, 1:2)
                if or(lang==["ja" "zh"])
                    tmp2 = tmp.root.children(3).content // <ul> list
                    RelCharWidth = uman_setRelCharWidth(tmp2, lang)
                end
                
                text = uman_xml2txt(tmp.root, " ")
                tmp2 = strsplit(text(1), ">")
                if grep(tmp2(1),"Scilab")~=[] then 
                    tmp2(1) = "Scilab"
                end
                tmp2 = stripblanks(tmp2)
                tmp2(find(tmp2=="")) = []
                if size(tmp2,"*")>1 & tmp2(1)==tmp2(2)
                    tmp2(2) = []
                end
            
                text(1) = strcat(tmp2," > ")+ascii(10)
                text(2) = []
                // Removing extra blank lines in deepest sub-blocks
                i = find(text==" ")
                text(i) = []
                // Displaying the summary page +++++++++++++++++++++++
                write(%io(2), text)
                xmlDelete(tmp)
            end
            // targeting the console
        end
        
    else // The required help file DOES NOT EXIST
         // -------------------------------------
        if grep(options, "w")==[]
            // Local macro:
            // -----------
            if localMacro~=""
                if grep(options,"c")~=[] then
                    clc()
                    mprintf("-> uman %s %s\n\n", fun0, options0)
                end
                head_comments(localMacro)  // 
            else // No dedicated page
                 // ----------------- 
                msg = _("uman"," %s: ""%s"" has no dedicated page.\n")
                mprintf(msg,"uman",fun0)
                if webRef~=[]
                    msg = _("uman %s w%s // should work.\n")
                    mprintf(msg,fun0, options0)
                end
                noContent = %t
            end
        end
    end

    if printConsole & grep(options,"j")==[] 
        diary([], "resume")               // http://bugzilla.scilab.org/12551
    end

    msg1 = ""

    // Using the helpbrowser
    // =====================
    // Must be after print in console => xml_grab() called and got r.parentsummary
    if grep(options,"g")~=[] then 
        global %helps
        if module=="scilab" ..
              | ( isATOMSinstalled & atomsIsLoaded(module) ) ..
              | find(convstr(%helps(:,2))==convstr(module))~=[] 
            // call the help browser in the chosen language
            if grep(options, "s")~=[]
                tmp = r.parentSummary
            elseif htmlName~=""
                tmp = basename(htmlName)
            end
            helpbrowser(%helps(:,1), tmp, LANG, %f)
            
        // fun = raw pattern => litteral search with the help browser            
        elseif module==""
            // For "g" and "w" options, the final target must be set
            // -----------------------------------------------------
            //  - if not "x": any equivalence is used
            //  - if "x": the initial fun0 is used
            if grep(options, "x")==[]
                helpbrowser(%helps(:,1), fun, LANG, %f)
                if fun~=fun0
                    msg1 = _("uman", " uman %s %s // used. Run: uman %s x%s // to really target ""%s""\n")
                    msg1 = msprintf(msg1, fun, options0, fun0, options0, fun0)
                end
            else
                helpbrowser(%helps(:,1), fun0, LANG, %f)
                tmp = uman_get_equiv(fun0, options, "scilabFirst") // | "ex2ex" | "exin2in"         
                if tmp~=fun0
                    msg1 = _("uman", " You may try: uman %s %s\n")
                    msg1 = msprintf(msg1, tmp, strsubst(options,"x", ""))
                end
            end
        elseif module~="" // module known but unloaded
            msg1 = _("uman", " [%s] module NOT LOADED => UNVAILABLE in HELP BROWSER\n")
            msg1 = msprintf(msg1, module)
        end
    end

    // Display of messages
    // ===================
    msg2 = ""
    if grep(options,"w")~=[] & module~="" & module~="scilab" & webRef(1)=="a"
        msg2 = " ["+module+"] "+..
               _("uman","module EXTERNAL   => ONLINE help page UNAVAILABLE")
    end
    tmp = " "+part("-~", ones(1,max(length([msg1 msg2]))/2).*.[1 2])
    if tmp~="" & (msg1~="" | msg2~="")
        msg0 = " uman " + fun0 + " " + options0 + " :";
        if ~noContent
            write(%io(2),tmp)
            if (msg0~=""), write(%io(2), msg0), end
        end
        if (msg1~=""), write(%io(2), msg1), end
        if (msg2~=""), write(%io(2), msg2), end
        if ~noContent, write(%io(2), tmp), end
    end
endfunction

// ===================================================================

// Copyright (C) 2013 - 2016  Samuel GOUGEON
// Utility function for uman()

function localMacro = uman_find_localMacro(fun)
    localMacro = ""
    ierr = execstr("Fun=" + fun + ";", "errcatch")
    if (ierr~=0 | typeof(Fun)~="function") & convstr(fun)==fun
        // fun may be the lower-cased name of another function 
        n = who("local")
        tmp = find(convstr(n)==fun)
        if tmp~=[]
            for f = tmp
                execstr("Fun  = "+n(f))
                if typeof(Fun)=="function"
                    ierr = 0
                    localMacro = n(f)
                    break
                end
            end
        end
    elseif ierr==0
        localMacro = fun
    end
endfunction

// ===================================================================

// Copyright (C) 2013 - 2016  Samuel GOUGEON
// Utility function for uman()

function [fun, options] = uman_updateQueryFromEquiv(fun, options)
    // Some equivalences have "£" inserted: prefixing chars are options to force
    //  By default, "s" is forced, meaning that the equivalence is designed
    //  to target a parent summary:
    if grep(fun,"£")~=[] then
        s = strsplit(fun, "£", 1)
        fun = s(2)
        if s(1)==""
            if grep(options, "s")==[]
                options = "s" + options
            end
        else
            options = s(1) + options
        end
        // Update priorities:
        if grep(options, "s")~=[]
            options = uman_strsubst(options, ["w" "a" "b" "d" "e" "h" ],"")
        end
        if grep(options, "u")~=[]
            options = uman_strsubst(options, ["w" "s" "a" "b" "d" "e" "h" ],"")
        end
    end
endfunction

// ===================================================================

// Copyright (C) 2013 - 2016  Samuel GOUGEON
// Utility function for uman()

// This function looks in all registered JHelpMap files for the entry with 
//  target="fun". If it is found, the path to the related HTML file is 
//  returned, as well as the name of the related module ("scilab" or external)  
function [htmlName, jhmLine] = uman_findItemInJHelpMaps(fun, jhmTab, mapDir, grepCase)
    htmlName = ""
    jhmLine = []
    for jhm = jhmTab'
        //doc = xmlRead(jhm)
        //e = xmlXPath(doc,"//mapID[@target="""+fun+"""]")
       doc = mgetl(mapDir+jhm(2))
       i = grep(doc, "/target="""+fun+"""/"+grepCase, "r")
       if length(i)>1
           i = grep(doc, "/target="""+fun+"""/", "r")
       end
       if i~=[] // We found the pattern!
           // Name of the .html file to be extracted?
           search = "/^.*?target="""+fun+""" url=""(.+?.html).*?"".*$/"+grepCase
           htmlName = uman_strsubst(doc(i), search, "$1","r")
           jhmLine = jhm
           break
       end
    end
endfunction

// ===================================================================

// Copyright (C) 2013 - 2016  Samuel GOUGEON
// Utility function for uman()

function uman_disp(man_contents, options)
    // Finally displays the formatted help text content into the console
    if or(part(LANG,[1 2])==["ja" "zh"])
        lengthFactor = 1.5
    else
        lengthFactor = 1
    end
    u = "----------------------------------------------------------"+..
        "----------------------------------------------------------"
    s = man_contents
    c = %io(2)
    
    t = s.path
    if grep(options,"u")==[]
        write(c, [ "" ; t ; part(".",ones(1,length(t))) ; ""])
        write(c, strcat(justify([s.refname s.refpurpose],"l")," - ","c"))
    else
        write(c, [ "" ; t ; part(".",ones(1,length(t)))])
    end

    
    if s.synopsis~=[] then
        t = s.synopsisTitle
        if grep(options, "u")==[]
            write(c, [ "" ; t ; part(u,1:length(t)*lengthFactor) ; " "+s.synopsis ])
        else
            write(c, [ convstr(t,"u") ; " "+s.synopsis ])
        end
    end

    if grep(options,"u")==[]
        // Loop over other sections, in the order they appear in the original page
        sections = man_contents.sections
        ns = size(sections,"*")
        
        has_params_section = %f // if false, description sections are displayed

        for is = 1:ns
            s = sections(is)
            tmp = s.type
            has_params_section = has_params_section | (tmp=="parameters" & s.contents~=[])

            if (or(tmp==["parameters" "see_also"]) | ..
                ((tmp=="description" & (grep(options,"d")~=[] | ~has_params_section))) | ..
                (tmp=="examples"    & grep(options,"e")~=[]) | ..
                (tmp=="history"     & grep(options,"h")~=[]))
                t = s.title
                if t~=""
                    write(c, [ "" ; t ; part(u,1:length(t)*lengthFactor)])
                end
                if s.contents~=[]
                    if t==""
                        write(c,["";""])
                    end
                    if tmp=="see_also"
                        s.contents = part(s.contents, 3:$)  // trimming starting stars
                    end
                    write(c, " "+s.contents)
                end
            end
        end
        // Bugs
        if grep(options,"b")~=[] 
            t = _("uman","Bugs")
            write(c, [ "" ; t ; part(u,1:length(t)*lengthFactor) ])
            write(c, [ _("uman","Known")+"  : " + man_contents.bugs_known ; ..
                       _("uman","Report")+" : " + man_contents.bug_report ])
        end
    end
endfunction

// ===================================================================

// Copyright (C)   2013 - 2016    Samuel GOUGEON
// Utility function for uman()

function r = uman_grab(htmlf, fun, LANG, options)
    // htmlf : HTML file name
    // fun : function's name or item
    // LANG: ##_## language string
    // options: options string from uman()
    // r. : structure with fields
    //  .fun .title  .path .refname .refpurpose .synopsisTitle .synopsis
    //  .paramTitle  .parameters .examples .seeAlsoTitle .seeAlso 
    //  .bug_report  .bugs_known
    //  .parentSummary

    lang = part(LANG,1:2)
    
    html = mgetl(htmlf)

//    search  = ["/<br>/" "/<meta .*?>/" "/<[^a-zA-Z!\/]/" ]
//    replace = ["<br/>"     ""             "&lt;"         ]
    search  = "/"+["<br>" "<meta .*?>" "<[^a-zA-Z!\/]" "&eacute;" "&egrave;" ]+"/"
    replace = ["<br/>"    ""           "&lt;"          "é"        "è"        ]
    html2 = uman_strsubst(strcat(html, " "), search, replace, "r")
    
    // Support to <sup> and <sub> tags
    search  = [ "/<sup>"+["([^\s]*?)" "(.*?)"]+"</sup>/" ..
                "/<sub>"+["([^\s]*?)" "(.*?)"]+"</sub>/"
               ]
    replace = ["\^$1" "\^{$1}" "_$1" "_{$1}"]
    html2 = uman_strsubst(html2, search(:), replace, "r")

    // URLs GET's & separators must be replaced with %26 in order to not break
    //  XML parsing. Replacing all cannot be done with TCL uman_strsubst because
    //  "&" has a special meaning in this TCL regsub context 
    // => using a while loop here: 
    src = ""
    while or(html2~=src) do
        src = html2
        html2 = uman_strsubst(src, ["|(href=""http:[^\""]*?)&([^\""]*?"")|" "|& |"],..
                                   ["$1%26$2" "&amp; "], "r")
    end

    doc = xmlReadStr(html2)
    
    // If the language is ja_JP: with must assess the fraction of content in
    // japanese vs latin, in order to tune the average character width used
    // to wrap long lines
    if or(lang==["ja" "zh"])
        tmp = doc.root.children(2).content  // Literal content of the body
        RelCharWidth = uman_setRelCharWidth(tmp, lang)
    end
    
    // Table of <pre.. </pre> sections (badly processed as XML)
    [s,e,preTable] = regexp(strcat(html,"££"),"%<pre[\s>].*?</pre>%mi")
    e = xmlXPath(doc,"//pre")
    for i = 1:size(s,"*")
        tmp = preTable(i)
        preTable(i) = uman_cleantxt(preTable(i),"tags|htmlent|style")
        // now we forge an id = full content cleaned
        preTableNoTag(i) = uman_strsubst(e(i).content, [" " "££"], ["" ""]) 
    end

    htmlcat = strcat(html, "££")
    
    r.fun = fun
    
    // Title
    e = xmlXPath(doc,"//title")
    r.title = e.content

    // Path in help
    // ------------
    e = xmlXPath(doc,"//span[@class=""path""]")
    tmp = strsplit(e.content, ">")
    if grep(tmp(1), "Scilab")~=[] then 
        tmp(1) = "Scilab"
    end
    tmp = stripblanks(tmp)
    tmp(find(tmp=="")) = []
    if tmp(1)==tmp(2)
        tmp(2) = []
    end
    r.path = strcat(tmp," > ")
    
    // Parent Summary (from path)
    tmp = "%<span class=""path"">.*?</span>%mi"
    [s, e, tmp] = regexp(html2, tmp)
    tmp2 = "%href="".*?""%mi"
    [s, e, tmp2] = regexp(tmp, tmp2)
    r.parentSummary = part(tmp2($), 7:$-6)

    // URL(report a bug)
    tmp = "%""http://bugzilla\.scilab\.org/enter_bug\.cgi.*?\""%"
    [s,e,m] = regexp(htmlcat, tmp)
    if m~="" then
        r.bug_report = strsubst(m,"""","")+"&short_desc="+fun
    else
        r.bug_report = "http://bugzilla.scilab.org/enter_bug.cgi"+..
        "?product=Scilab%20software&component=Documentation%20pages"+..
        "&short_desc="+fun
    end
    // URL(known bugs)
    // BugzillaSearch is defined in the calling level
    r.bugs_known = strsubst(BugzillaSearch,"£",fun)
    
    // refname(s)
    e = xmlXPath(doc,"//h1[@class=""refname""]")
    r.refname = xmlAsText(e)'
    
    // refpurpose
    e = xmlXPath(doc,"//p[@class=""refpurpose""]")
    r.refpurpose = uman_cleantxt(xmlAsText(e)', "blanks")
        // Some pages presenting multiple equivalent functions do not repeat
        // the short description for last aliases 
        // (printf.. http://bugzilla.scilab.org/13748)
        // We must complete them here:
    tmp =  r.refpurpose
    if size(tmp,1)<size(r.refname,1) & r.refpurpose($)~=[] then
        r.refpurpose = [tmp ; emptystr(size(r.refname,1)-size(tmp,1),1)+r.refpurpose($)]
    end
    
    // synopsis title
    CS = struct("en","Syntax",..
                "fr","Usages",..
                "ja","呼出し手順", ..
                "pt","Seqüência de Chamamento",..
                "de","Aufruf", ..
                "ru","Последовательность вызова",..
                "zh","调用序列",..
                "fa","دنباله‌ی فراخوانی")
    if isfield(CS,lang) then
        r.synopsisTitle = CS(lang)
    else
        e = xmlXPath(doc,"//div[@class=""refsynopsisdiv""]")
        if e.size>0 then
            r.synopsisTitle = e(1).children(1).content
        else
            r.synopsisTitle = _("uman", "Syntax")
        end
    end

    // Synopsis
    //s = xmlXPath(doc,"//div[@class=""synopsis""]")
    [s,e,M] = regexp(htmlcat, "%<div class=""synopsis""><pre>(.*?)</pre>%")
    tmp = []
    for m = M'
        s = strsplit(uman_cleantxt(m,"tags|htmlent|style"),"££")
        tmp = [tmp ; s ]
        if size(s,1)>1 then
            tmp = [tmp ; ""]
        end
    end
    tmp = strsubst(tmp, "&amp; ", "&")
    if tmp($)=="" then // removing the last blank line
        tmp($) = []
    end
    if size(r.refname,1)==1 then    // Removing blank lines
        tmp(find(tmp=="")) = []
    end
    // When a page has several xml:id, get syntaxes matching only the pattern
    if tmp~=[] then
        i = grep(" " + tmp, "/[^a-zA-Z]" + fun + "/","r")
        if i~=[] then
            tmp = tmp(i)
        end
    end

    // Synopses post-processing: alignment (if required)
    global umanAlignSyntaxes
    if umanAlignSyntaxes==[] then
        umanAlignSyntaxes = "r"     // default alignment mode
    end
    if size(tmp,1)>1 & or(umanAlignSyntaxes==["l" "r"]) then
        // Aligning  "= fun(..)"
        q  = strstr(tmp, "= "+fun+"(");
        Lq = length(q);
        k  = find(Lq>0);
        hk = ""
        if k~=[] then
            hk = justify(strncpy(tmp(k), strcspn(tmp(k), "=")), umanAlignSyntaxes);
            tmp(k) = hk + " " + q(k)
        end
        // Aligning remaining "fun"
        k2 = find(Lq==0);
        if k2~=[] then
            hwidth = length(hk(1));
            s = [];
            for i = 1:length(k2)
                istart = regexp(tmp(k2(i)), "|"+fun+"|");
                if istart>1 then
                    s = [s ; strsplit(tmp(k2(i)), istart-1)'];
                else
                    s = [s ; ["" tmp(k2(i))]]
                end
            end
            if k~=[]
                s = [s ; [part(".",ones(1,hwidth+3)) ""]];
                s(:,1) = justify(s(:,1), umanAlignSyntaxes)
                s($,:) = []
            else
                s(:,1) = justify(s(:,1), umanAlignSyntaxes)
            end
            tmp(k2) = strcat(s, "", "c")
        end
    end
    r.synopsis = tmp
    
    if grep(options, "u")==[]
        
        // REFSECTIONS
        // ===========
        refsections = xmlXPath(doc,"//div[@class=""refsection""]")
        nrs = length(refsections)
    
        // MAIN LOOP over SECTIONS of all TYPES
        // ------------------------------------
        for i = 1:nrs
            rs = refsections(i)
            secTitle = xmlXPath(rs, "./h3[@class=""title""]")
            if size(secTitle,"*")>0
                tmp = uman_cleantxt(secTitle(1).content, "blanks")
                [secType, typeName] = uman_typeof_section(tmp, lang)
                r.sections(i).title = typeName
                
                contents = uman_xml2txt(rs, "")
                contents(1) = []     // suppressing the embedded title
            else
                secType = "description"
                r.sections(i).title = ""
                contents = uman_xml2txt(rs, "")
            end
            r.sections(i).type  = secType
            if secType=="examples" then
                contents = uman_strsubst(contents, "/\[IMAGE\: .*?\]/","","r")
                contents = strsubst(contents, "\&", "&")
            elseif secType=="see_also"
                k = find(stripblanks(contents)=="")
                contents(k) = []
            end
            while contents(1)=="" | grep(contents(1),"/^\s*$/","r")~=[]
                contents(1) = []
            end
            while contents($)=="" | grep(contents($),"/^\s*$/","r")~=[]
                contents($) = []
            end
            
            r.sections(i).contents = contents
        end
    end

    xmlDelete(doc)
endfunction

// ===================================================================
// Copyright (C)  2013 - 2016   Samuel GOUGEON
// Utility function for uman()

function [options, LANG] = uman_splitOptionsLang(options)
    i = strindex(options, "l")
    LANG = ""
    if i~=[] then
        LANG    = part(options, (i(1)+1):(i(1)+2))
        if LANG==""
            LANG = "en"
        end
        options = uman_strsubst(options, ["/l.._../i" "/l../i"], ["" ""], "r")
    end
endfunction

// ===================================================================
// Copyright (C)  2013 - 2016   Samuel GOUGEON
// Utility function for uman()

// Japanese and Chinese characters are wider than latin ones.
// This function estimates the average char relative width according to the
//  estimated amount of Japanese or Chinese characters in the passed nakedText,
//  over all characters (mixed with latin ones).
// This relative width is needed when wrapping lines.
function relCharWidth = uman_setRelCharWidth(nakedText, lang)
    tmp = uman_cleantxt(nakedText, "blanks")  // removal of consecutive blanks
    Nchar = length(tmp)
    Nascii = length(ascii(tmp))
    if or(lang==["ja" "zh"])
        // 100% japanese => Nascii~3*Nchar  <=> 1 jaChar ~ 1.4 latinChar
        if Nascii/Nchar>1.5     // detection threshold
            relCharWidth  = 1.4
        else
            relCharWidth  = 1
        end
    end
endfunction

// ===================================================================

// Copyright (C)  2013 - 2016   Samuel GOUGEON
// Utility function for uman()

function [secType, typeName] = uman_typeof_section(secTitle, lang)
    // id vs titles for sections    
    langs = [ "en" "de" "fr" "ja" "pt" "ru" "zh" "fa"]
    ID = ["parameters"              "description"  "examples" ]
    en = ["Parameters,Parameter,arguments"    "Description"  "Examples,example"]
    de = ["Parameter"               "Beschreibung" "Beispielen,beispiel,beispiele"]
    fr = ["Paramètres"              "Description"  "Exemples,exemple"]
    ja = ["設定,引数,パラメータ"       "説明"         "例"]
    pt = ["Parâmetros,Argumentos"   "Descrição"    "Exemplos,exemplo"]
    ru = ["Аргументы,Параметры"     "Описание"     "Примеры,Примерами,ример,примеров"]
    zh = ["設置"                    "描述"          "示例,例子"]
    fa = ["آرگومان‌ها"               "توصیف"        "مثال‌ها"]

    ID = [ ID "see_also"    "history"   ]
    en = [ en "See Also"    "History"   ]
    de = [ de "Siehe auch"  "Historische"]
    fr = [ fr "Voir aussi"  "Historique"]
    ja = [ ja "参照"         "履歴"      ]
    pt = [ pt "Ver também"  "Histórico" ]
    ru = [ ru "Смотрите также" "История"]
    zh = [ zh "参见"         "历史的"    ]
    fa = [ fa "همچنین ببینید" "تاریخچه"  ]  // "See also" in Farsi (bug to be fixed)

    k = find(langs==lang)
    if k==[] then
        lang = "en"
    end
    execstr("Ref = "+lang)
    secType = "description" // default
    typeName = secTitle     // default
    L = length(secTitle)
    secTitle= convstr(secTitle)
    
    ref = convstr(Ref)
    found = %f
    while ~found
        for i=1:size(ref, 2)
            tmp  = tokens(ref(i), ",") 
            tmp2 = tokens(Ref(i), ",")
            [j,k] = grep(secTitle, tmp)
            if j~=[] & (i~=1 | or((L-length(tmp(k)))<3)) then
                secType = ID(i)
                if or((L-length(tmp(k)))<3)
                    typeName = tmp2(1)
                end
                found = %t
                break
            end
        end
        tmp = convstr(en)
        if found | and(ref==tmp), break, end
        ref = tmp
    end
endfunction

// -------------------------------------------------------------------

// Copyright (C)  2013 - 2016   Samuel GOUGEON
// Utility function for uman()
function out = uman_cleantxt(txt, cleanjobs)
    // cleanjobs: "tags|htmlent|catlines|style|blanks"
    cleanjobs = tokens(cleanjobs,"|")'
    out = txt
    for cleanjob = cleanjobs
        select cleanjob
        case "tags"
             out = uman_strsubst(out, "/<.*?>/", "", "r")
         case "catlines"
             for i = 1:size(out,"*") 
                out(i) = strcat(stripblanks(strsplit(out(i),"££"))," ")
             end
         case "htmlent"
             patterns = ["&#0034;" "&#0038;" "&#0039;" "&#0060;" "&#0062;" "&gt;" "&lt;" "&quot;" ]
             replaces = [ """"       "\&"       "''"      "<"       ">"       ">"   "<"   """"    ]
            out =  uman_strsubst(out, patterns , replaces)
         case "style"
            out =  uman_strsubst(out, "/([^ \=<>~])=([^ \=''""])/", "$1 = $2", "r")
            out =  uman_strsubst(out, "/([^\s]);([^\s])/", "$1; $2", "r")
        case "blanks"
            out = uman_strsubst(out, "/((\s{2,}))/", " ", "r")
        end
    end
endfunction

// -------------------------------------------------------------------

// Copyright (C)  2013 - 2016   Samuel GOUGEON
// Utility function for uman()

function out = uman_wraplines(text, ncol)
    // LANG must be set in the calling environnement
    if text==[] then
        out = []
        return
    end
    if or(LANG==["ja_JP" "zh_CN"]) then
        // RelCharWidth is defined in uman_grab() in the calling environnement
        nchar = floor(ncol/RelCharWidth)  // because symbols are wider than latin ones
    else
        nchar = ncol
    end
    nchar = nchar-1
    text = stripblanks(text)
    Sep = " "
    if LANG=="ja_JP" then
        Sep = [Sep "。" "・"]
//        [T, sep] = strsplit(text, Sep)
//        sep($+1) = ""
//    else
//        if text~=[]
//            T = tokens(text," ")    // 30x faster than strsplit...
//            sep = emptystr(T)+" "
//        else
//            T = []
//            sep = ""
//        end
    end
    [T, sep] = strsplit(text, Sep)
    sep($+1) = ""
    LT = length(T)+1
    out = []
    line = ""   // current line
    lineL = 0   // lineLength
    for i = 1:size(T,"*")
         if lineL+LT(i)<=nchar | line==""
             line = line + T(i) + sep(i)
             lineL = lineL + LT(i)
         else
             if lineL+LT(i)>nchar & LANG=="ja_JP" & grep(line,"/(—\s|[^。・\s])$/","r")~=[]
                 line = line + T(i) + sep(i)
                 out = [out ; line]
                 line = ""
                 lineL = 0
             else
                 out = [out ; line]
                 line = T(i) + sep(i)
                 lineL = LT(i)
             end
         end
    end
    if stripblanks(line,%t)~="" then
        out = [out ; line]
    end
    out = stripblanks(strsubst(out, "¤"," "))
endfunction

// -------------------------------------------------------------------

// Copyright (C)  2013 - 2016   Samuel GOUGEON
// Utility function for uman()

function newT = uman_wraptable(T, nchar)
// * Multicolumn table with long unwrapped lines (ex: uman atomsSetConfig ac )
//   .nchar = nombre total de caractères
//   . nc = nombre total de colonnes
//   . C1 = colonnes dont la largeur est < nchar/nc+2
//     n1 = nombre de colonnes C1
//     w1 = largeur cumulée des colonnes étroites C1
//     C2 = colonnes dont la largeur est >= nchar/nc+2
//     n2 = nombre de colonnes C2
//     C3 = colonnes parmi les C2 dont la largeur est < (nchar-w1)/n2
//     n3 = nombre de colonnes C3
//     w3 = largeur cumulée des colonnes demi-larges C3
//     C4 = colonnes parmi les C2 dont la largeur est >= (nchar-w1)/n2
//          = colonnes à wrapper.
//     n4 = nombre de colonnes C4
//          largeur de wrappage wW = (nchar-w1-w3)/n4 - 2
    nc = size(T,2)

    // To compute the columns widths, we must not take into account rows that
    //  have colspan cells (cell starting with "#§" where # is the colspan range)
//    colspanLines = grep(strcat(T,"","c"), "/\d+§/", "r") // rows having at least one colspan
//    tmp2 = T
//    tmp2(,:) = []
//    cw = max(length(tmp2),"r") // column widths
    cw = max(length(T),"r") // column widths
    
    C1 = find(cw < (nchar/nc+2))
    n1 = length(C1)
    w1 = sum(cw(C1))
    C2 = find(cw >= (nchar/nc+2))
    n2 = length(C2)
    if n2>0 then
        C3 = find(cw(C2) < (nchar-w1)/n2)
        C3 = C2(C3)
        C4 = find(cw(C2) >= (nchar-w1)/n2)
        C4 = C2(C4)
    else
        C3 = []
        C4 = []
    end
    n3 = length(C3)
    w3 = sum(cw(C3))
    n4 = length(C4)
    
    // Some wide columns to be wrapped exist
    indent = ""
    if n4>0 then
        wW = (nchar-w1-w3)/n4 - length(indent)
        newT = []
        // Loop over lines of the table
        for i = 1:size(T,1)
            if scilab6, c = {}, else c = cell(), end
            // Loop over columns of the line
            for j=1:size(T,2)
                if or(j==C4)
                    tmp = uman_wraplines(T(i,j), wW)
                    if size(tmp,1)>1
                        tmp(2:$) = indent + tmp(2:$)
                    end
                else
                    tmp = T(i,j)
                end
                if scilab6, c(j) = {tmp}, else c(j).entries = tmp, end
                nL(j) = size(tmp,1)
            end
            nnL = max(nL)
            BoL = []    // new Block of (wrapped) Lines
            for j = 1:size(T,2)
                if scilab6
                    // execstr used to be Scilab 5 parsable
                    execstr("tmp = c{j}") 
                else
                    tmp = c(j).entries
                end
                // useless: justify() will remove leading blanks
                // if size(tmp,1)>1
                //     tmp(2:$,:) = "." + tmp(2:$,:)
                // end
                BoL = [BoL [tmp ; emptystr(nnL-nL(j),1)]]
            end
            newT = [newT ; BoL]
        end
    else
        newT = T
    end
endfunction

// -------------------------------------------------------------------

// Copyright (C)  2013 - 2016   Samuel GOUGEON
// Utility function for uman()

function [txt, inline] = uman_xml2txt(node, lineHead)
    // Upstream variables used: options, secType
    //
    // Pieces of HTML code (template to decode):
    //<code..>bla bla</code>   : =>  [bla bla]  // cancelled
    //<a href=""..>bla bla</a> : => _{bla bla}  // cancelled
    // <ul class="itemizedlist">
    //       <li><p class="para">..</p> : => nouvelle ligne à chaque <p..>
    //           <p..>...</p>
    //       </li>
    // </ul>
    // <img ..> among which 
    //   <img src="ScilabNote.png"/> and <img src="ScilabWarning.png"/>
    //   <img src="ScilabTip.png"/> and <img src="ScilabCaution.png"/>
    //   <img src="ScilabImportant.png"/>

    RelCharWidth = RelCharWidth // defined in the calling environnement
    inline = %F
    thistxt = []
    thisline = []
    name = convstr(node.name)
    nc = node.children.size     // number of children
    
    // Max line width limited to umanMaxLineWidth or by default to 90 chars
    cw = lines()
    tmp = 90
    global umanMaxLineWidth
    tmp2 = umanMaxLineWidth
    if tmp2~=[]
        if type(tmp2)~=1 | ~isreal(tmp2) | tmp2<50
            tmp = 90
        else
            tmp = tmp2
        end
    end
    cw = min([cw(1) tmp])  
    // ---------------------------------------------------------------
    if name=="code" then
        txt = strsubst(uman_cleantxt(node.content, "style|blanks"), " ", "¤")
        inline = %T
        
    elseif or(name==["text" "a" "i"]) then
        txt = uman_cleantxt(node.content, "blanks")
        inline = %T
        
    // ---------------------------------------------------------------
    elseif or(name==["b" "em"]) then
        txt = uman_cleantxt(node.content,"blanks")
        global umanNoStyle
        if ~umanNoStyle then
            if  name=="em" then
                txt = "/"+txt+"/"
            else
                txt = convstr(txt,"u")
            end
        end
        inline = %T
        
    // ---------------------------------------------------------------
    elseif name=="varname"
        txt = "[" + uman_cleantxt(node.content, "blanks") + "]"
        inline = %T

    // ---------------------------------------------------------------
    //    elseif name=="a" 
    //        // <a> over complex content will ignore content's format :
    //        //txt = "_{" + node.content + "}"
    //        txt = node.content
    //        inline = %T

    // ---------------------------------------------------------------
    elseif or(name==["h3" "div" "p" "span" "li" "dd"])
        if name=="div" & node.attributes.class=="programlisting" 
            k = find(preTableNoTag==strsubst(node.content," ",""))
            if k~=[]
                //tmp = strsplit(preTable(k(1)),"££")
                //tmp = part(".", ones(1,40)) // "......................"
                //txt = [ tmp; strsplit(preTable(k(1)),"££") ; tmp ; ""]
                txt = [ strsplit(preTable(k(1)),"££") ; ""]
            else
                txt = ""
            end
        elseif name=="div" & node.attributes.class=="screen" 
            txt = ""
            k = find(preTableNoTag==strsubst(node.content," ",""))
            if k~=[]
                tmp = strsplit(preTable(k(1)),"££")
                if tmp(1)=="", tmp(1)=[], end
                if tmp($)=="£", tmp($)=[], end
                if grep(tmp($),"/^\s*$/","r")~=[], tmp($)=[], end
                tmp2 = max(length(tmp))
                tmp = ":  "+part(tmp,1:tmp2)+" :"
                tmp2 = part(".",ones(1,tmp2+5))
                txt = [ tmp2; tmp; tmp2 ]
            end
        else
            // Wrapping width
            // wW = cw-length(lineHead)-3    
            wW = cw - length(lineHead)
            if wW<30, wW = 30, end            // minimal width = 30 chars
            //
            forceInLine = (name=="li" & isdef("simpleContent", "n"))
            if name=="li" & forceInLine
                tmp = xmlXPath(node, "./ul")  // are there any nested sub-list?
                if size(tmp, "*")>0         //   yes:
                    forceInLine = %f
                end
            end

            for i = 1:nc
                [ tmp, inL ] =  uman_xml2txt(node.children(i), lineHead)
                if inL | forceInLine then
                    if thisline==[]
                        thisline = strcat(tmp, " ")
                    else
                        thisline = thisline + strcat(tmp, " ")
                    end
                else 
                    tmp2 = uman_wraplines(uman_cleantxt(thisline,"blanks"), wW)
                    thistxt = [ thistxt ; tmp2 ; tmp ]
                    thisline = []
                end
            end
            if thisline~=[]
                tmp2 = uman_wraplines(uman_cleantxt(thisline,"blanks"), wW)
                txt = [ thistxt ; tmp2 ]
            else
                txt = thistxt
            end
            if name=="span" & size(txt,1)==1 then
                inline = %T
            end
        end
        
    // ---------------------------------------------------------------
    elseif name=="pre"
        // preTable, preTableNoTag = external variables 
        k = find(preTableNoTag==strsubst(node.content, " ", ""))
        txt = strsplit(preTable(k(1)), "££")

    // ---------------------------------------------------------------
    elseif name=="img"  // ex: <img src='jarsci:./PlotSparse_1.png'/>
        // works with Scilab pages. Untested with external packages
        tmp = node.attributes.src;
        tmp = tokens(tmp,"/")
        txt = "[IMAGE: "+tmp($)+"]"
        inline = %t

    // ---------------------------------------------------------------
    elseif name=="dl"     // definitions list
        // <dl> : definitions list : ignorer content ; traiter les children
        //   <dt><span class="term">..</span>      : definition term
        //       <dd><p class="para">...</p></dd>  : definition definition
        //   </dt>
        // </dl>
        // Processing all <dt> entries
        terms = []     // matrix of terms
        defs  = cell() // cell of related definitions
        i = 1    // n° of <dl>'s children
        it = 0   // n° of terms
        maxL = 8   // maximal length of "alignable" terms
        minL = 5    // minimal indentation of lines for the bloc
        while i<=nc
            // only <dt>..</dt> should be considered:
            while node.children(i).name~="dt" & i<nc
                i = i+1
            end
            dt = node.children(i)
            // Searching for the <span class="term">..</span> tag:
            j = 1
            sp = dt.children(1)
            while sp.name~="span" & j<dt.children.size
                j = j+1
                sp = dt.children(j)
            end
            it = it+1            
            tmp = stripblanks(sp.content)
            // trimming any ending ":" (as in the ode page)
            L = length(tmp)
            if L>0 & part(tmp,L)==":"
                terms(it)  = part(tmp,1:L-1)
            else
                terms(it) = tmp
            end
            
            // looking for the forthcoming <dd> section:
            sdt = dt.children.size
            j = min([2,sdt])   // n° of <dt>'s children
            while j<sdt & convstr(dt.children(j).name)~="dd" 
                j = j+1
            end
            //dd = dt.children(j) should be <dd><p class="para">...</p></dd>
            // local header length (estimated)
            k = find(length(terms)<maxL)
            if k~=[]
                hL = max(length(terms(k)))+3
            else
                hL = minL
            end
            ddtxt = uman_xml2txt(dt.children(j), lineHead+part(" ",ones(1:hL)))
            if scilab6
                execstr("defs{it} = ddtxt")
            else
                execstr("defs(it).entries = ddtxt")
            end
            // nothing is expected between </dd> and the embedding </dt>
            i = i+1
        end

        // Building the whole set of <dt> entries
        // --------------------------------------
        // For short terms, the description is appended to term's line :
        // Setting the standard width of items label field
        I = find(length(terms)<maxL)
        if I~=[] then
            nC = max(length(terms(I))) + 3  // +3 for " : "
            terms(I) = justify(terms(I),"l")
        else
            nC = minL
        end
        // Setting the local blank header
        header = part(" ", ones(1:nC))
        
        // Setting the standard width of item Description field
        wD = max([30, cw - length(lineHead+header) ])
        
        // Loop on items
        for L = 1:size(terms,1)
            wD_line1 = wD - max([ 0 (length(terms(L))+3-nC)])
            if scilab6
                execstr("tmp = defs{L}")
            else
                execstr("tmp = defs(L).entries")
            end
            if size(tmp,1)<2 then
                tmp2 = uman_wraplines(tmp, wD_line1+1)
                thistxt =[thistxt; terms(L)+" : "+tmp2(1)]
                if tmp~=tmp2(1) then
                    tmp2 = uman_wraplines(strcat(tmp2(2:$)," "), wD)
                    thistxt  = [thistxt; ""; header + tmp2]
                end
            else
                if wD_line1 < wD
                    // The first line is kept clear
                    thistxt =[thistxt; terms(L)+" : " ;  header+tmp ]
                else
                    thistxt =[ thistxt ; terms(L)+" : "+tmp(1); header + tmp(2:$)]
                end
            end
            if size(tmp,1)>1 & stripblanks(thistxt($))~="" then
                thistxt($+1) = ""
            end
        end
        txt = " "+thistxt
    // ---------------------------------------------------------------
    elseif or(name==["ul" "ol"])
        // Processing all <li> entries :
        litab = []
        for i = 1:nc
            li = node.children(i)
            
            // Summary or See also list: preprocessing
            //  If the item's title has white spaces, it isn't the item's id 
            //   => if it has .html & isn't a help section: 
            //      replace the title with the id = html filename
//            if (grep(options, "s")~=[] & part(htmlName,1:8)=="section_") | secType=="see_also"
            if grep(options, "s")~=[] | secType=="see_also"
                tmp = li.content
                tmp2 = strsplit(tmp,"—",1)
                tmp2 = stripblanks(tmp2(1)) // item's title
                // We get the related id: item_id.html => item_id
                if length(li.children)>0
                    href = li.children(1).attributes.href
                else
                    href = ""
                end
                if grep(href,".html")~=[] 
                    // not the case for Scilab entries quoted from external modules
                    // that have href="scilab://item" instead of href="item.html"
                    id = strsplit(href,".",1)
                    id = id(1)
                    if id~=[] & part(id,1:8)~="section_"
                        // We replace the title with the id:
                        tmp4 = strstr(tmp, "—") // get the short description
                        if length(tmp4)<3
                            // if short description empty =>put the title in it
                            li.content = id+" — "+tmp2 
                        else
                            li.content = id+" "+tmp4
                        end
                    end
                end
            end
            // --------------
                
            tmp = uman_xml2txt(li, "  "+lineHead)
            if tmp~=[]
                if name=="ul"
                    tmp(1) = "* "+tmp(1)
                else
                    tmp(1) = sprintf("%2d",i)+". "+tmp(1)
                end
            end
            if size(tmp,1)>2 & stripblanks(tmp($))~="" then
                tmp($+1) = ""
            end
            if size(tmp,"*")>1
                tmp(2:$) = "  "+tmp(2:$)
            end
            litab = [ litab ; tmp ]
        end
        if size(litab,1)>8 then
            litab = [""; litab]
        end
        txt = " "+litab
        
        // Post-processing for the list of summaries, in order to align the "--"
        if grep(options, "s")~=[] | secType=="see_also"
            K = grep(txt, "/^ \*.*?—/","r")
            Kw = grep(txt, "/^   [^\s\*]/", "r")  // wrapped line
            nL = size(txt,1)
            if K~=[] then
                txt1 = stripblanks(txt(K))
                s = strcspn(txt1,"—")            // locating the "—" separator
                ms = mean(s)
                ss = stdev(s)
                p = min([18, round(ms+ss)])
                if ss<7 then
                    p = min([18, round(ms+1.5*ss)])
                end
                j = find(s>0 & s<p)       // # of lignable lines
                t = []
                for m = j
                    t = [ t ; strsplit(txt1(m),s(m))']
                end
                p = min([18, max(length(t(:,1)))+1])
                if t~=[]
                    t = justify(t(:,1), "l") + " " + t(:,2)
                end
                txt1(j) = t
                txt(K) = txt1
                // Shifting wrapped lines
                head = part(" ", ones(1,p+3))
                ww = max([30, cw - length(lineHead)-p]) // wrapping width
                
                newtxt = []
                line = ""
                for i = 1:size(txt,"*")
                    if or(i==Kw) 
                        line = line + " "+ stripblanks(txt(i))
                    else
                        if line~=""
                            // we process the registered line
                            tmp = uman_wraplines(line, ww+p+3)
                            if tmp~=[]
                                newtxt = [ newtxt ; " "+tmp(1) ]
                            end
                            if size(tmp,1)>1
                                tmp = uman_wraplines(strcat(tmp(2:$), " "), ww)
                                newtxt = [newtxt ; head + tmp ]
                            end
                            line = " "
                        end
                        if or(i==K)
                            line = " "+txt(i)
                        else
                            newtxt = [newtxt ; txt(i)]
                        end
                    end
                end
                if line~=" "
                    // we process the registered line
                    tmp = uman_wraplines(line, ww+p+3)
                    newtxt = [ newtxt ; " "+tmp(1) ]
                    if size(tmp,1)>1
                        tmp = uman_wraplines(strcat(tmp(2:$), " "), ww)
                        newtxt = [newtxt ; head + tmp ]
                    end
                end
            else
                newtxt = txt
            end
            txt = newtxt
        end
//     ---------------------------------------------------------------
    elseif or(name==["table" "tbody"])
        // <note> and <warning> are in a table starting with an image:
        //   <table><tr><td valign="top"><img src="ScilabNote.png"/>
        tmp = node.children(1).children(1).children(1)
        if tmp.name=="img"
            if grep(tmp.attributes.src, ["ScilabNote.png" "ScilabTip.png"])~=[]
                txt = convstr(_("Note"),"u")+" : "
            elseif grep(tmp.attributes.src, ["ScilabWarning.png" "ScilabCaution.png"])~=[]
                txt = convstr(_("Warning"),"u")+" : "
            elseif grep(tmp.attributes.src, "ScilabImportant.png")~=[]
                txt = "(!!): "
            else
                txt = ""
            end
            txt = txt + uman_cleantxt(node.content+" ","blanks")
            txt = uman_wraplines(txt, max(30, cw-length(lineHead)))
            return
        end
        a = node.attributes
        TABLE = %t
        Array = []
        for i = 1:nc        // Loop over rows
            row = uman_xml2txt(node.children(i)) 
            if i>1 
                // missing cells?
                if size(row,2)< size(Array,2)
                    row = resize_matrix(row, 1, size(Array,2))
                elseif size(row,2)> size(Array,2)
                    Array = resize_matrix(Array, 1, size(row,2))
                end
            end
            Array = [Array ; row ]
        end
        // If there are empty (or blank) columns => we remove them
        L = length(stripblanks(strcat(Array,"","r")))
        Array(:, find(L==0)) = []
        Array = uman_wraptable(Array, max(30, cw-length(lineHead))-1)
        
        Array = justify(Array, "l")
        if a.border~=[] & a.border~="0"
            Array = strcat(Array,"| ","c")
            Array = "| " + Array + " |"
            hline = part("-",ones(1,length(Array(1,:))))
            txt = " " + [hline ; Array ; hline ]
        else
            Array = strcat(Array," ","c")
            if size(Array,1)==1 then
                txt = stripblanks(Array)
            else
                txt = " " + Array
            end
            
        end
    // ---------------------------------------------------------------
    elseif name=="tr"
        Row = node.children
        row = []
        if nc>1
            // We assume that <td> or <th> cells have simple contents:
            for i = 1:nc
                tmp = uman_cleantxt(Row(i).content+" ","blanks")
                if Row(i).name=="th"
                    tmp = convstr(tmp,"u")
                end
                row($+1) = tmp
            end
            txt = row.'
        else
            txt = uman_xml2txt(Row(1), lineHead)
        end
    // ---------------------------------------------------------------
    elseif or(name==["td" "th"])
        txt = uman_cleantxt(node.content+" ","blanks")
        // Tagging any colspan in head of the cell
        k = find(convstr(xmlName(node.attributes))=="colspan")
        if k~=[]
            txt = node.attributes(k) + "§" + txt
        end
    // ---------------------------------------------------------------
    else 
        txt = "<"+name+">"+uman_cleantxt(node.content+" ","blanks")+"</"+name+">"
    end
endfunction

// ------------------------------------------------------------------

// Copyright (C)  2013 - 2016   Samuel GOUGEON
// Utility function for uman()
function uman_openURL(url, msg)
    if getos()=="Windows"
        winopen(url)
    elseif getos()=="Linux"
        unix("xdg-open """+url+"""")
    elseif getos()=="Darwin"         // Mac OS
        unix("open """+url+"""")
    else
        warning(msg)
    end
endfunction

// ------------------------------------------------------------------

// Copyright (C)  2013 - 2016   Samuel GOUGEON
// Utility function for uman()
function [item , options] = uman_get_equiv(item, options, equivType)
// item: the item to search
// List of equivalences or nearest redirections
//  column #1: external item (Octave, etc...).
//  column #2: Scilab redirection (help's filename).
//             "£" prefix = options to be passed when calling uman (default = s)
// equivType : "shortcuts" | "ex2in" | "ex2ex" | "exin2in"

    // Redirections inner => inner:
    // The left entry must not be a legal Scilab entry
    // The right entry must be a legal Scilab entry
    shortcuts = [
      "colors"   "£color"           // inner redirection | "£color_list" si "w"
      "datatypes" "d£type"          // inner redirection with "d" forced
      "files"   "£listfiles"
      "plotting" "£Graphics"
      "gui"     "£addmenu"
      "hdf5"    "£h5read"           // http://www.hdfgroup.org/HDF5/
      "history" "£gethistoryfile"   // inner summary
      "keys"    "d£console"         // inner redirection
      "latex"   "d£math_rendering_features_in_graphic" // inner redirection
      "lenght"  "length"            // common mis-spelling
      "signal"   "£detrend"
      "stats"    "£reglin"
      "trigo"    "£cos"
      "variables" "£who"
      "windows"  "£winopen"
      "xml"     "£xmlRead"
    ]

    // External => Scilab redirections
    // In ScilabFirst mode as long as the left entry is not a legal Scilab entry
    ex2in = [
      "all"     "and"
      "angle"   "atan"
      "any"     "or"
      "arrayfun" "feval"
      "atan2"   "atan"
      "axes"    "newaxes"
      "axis"    "gca"
      "balance" "balanc"
      "bicg"    "conjgrad"
      "bicgstab" "conjgrad"
      "box"     "gca"
      "caxis"   "colorbar"
      "cgs"     "conjgrad"
      "chebwin" "window"
      "cla"     "delete"
      "class"   "typeof"
      "clear global"    "clearglobal"
      "closereq" "xdel"
      "colormap" "jetcolormap"
      "compan"   "companion"
      "contourc" "contour2di"
      "contour3" "contour"
      "copyobj" "copy"
      "cputime" "timer"
      "csvread" "csvRead"
      "dbclear" "delbpt"
      "dbcont"  "resume"
      "dbquit"  "abort"
      "dbstack" "where"
      "dbstatus" "dispbpt"
      "dbstop"  "setbpt"
      "dbup"    "return"
      "deal"    "brackets"
      "deblank" "stripblanks"
      "demo"    "demo_gui"
      "dlmwrite" "fprintfMat"
      "echo"    "mode"
      "eig"     "eigs"
      "ellipj"  "amell"
      "ellipke" "percentk"
      "eps"     "percenteps"
      "errorbar" "errbar"
      "errordlg" "messagebox"
      "false"   "percentf"
      "fclose"  "mclose"
      "feof"    "meof"
      "fftn"    "fft"
      "fgetl"   "mgetl"
      "fgets"   "mgetl"
      "fill"    "xfpolys"
      "flipud"  "flipdim"
      "fliplr"  "flipdim"
      "fopen"   "mopen"
      "fread"   "mget"
      "frewind" "mseek"
      "fwrite"  "mput"
      "fzero"   "fsolve"
      "gallery" "testmatrix"
      "gammainc" "cdfgam"
      "ginput"  "locate"
      "grid"    "xgrid"
      "gtext"   "legends"
      "help"    "uman"          // Octave
      "helpdlg" "messagebox"
      "helpwin" "help"
      "hggroup" "glue"
      "hgload"  "load"
      "hgsave"  "save"
      "hist"    "histplot"
      "histcounts" "histc"
      "hold"    "axes_properties"
      "housh"   "householder"   // Octave
      "horzcat" "cat"
      "i"       "percenti"
      "ifft2"   "ifft"
      "ifftn"   "ifft"
      "imagesc" "Matplot1"
      "inf"     "percentinf"
      "invfeqz" "frfit"
      "iscell"  "typeof"
      "iscellstr" "typeof"
      "isgraphics" "is_handle_valid"    // Octave
      "ishghandle" "is_handle_valid"    // Octave
      "isinteger" "type"
      "isjava"  "typeof"
      "iskeyword" "getscilabkeywords"
      "ismac"   "getos"
      "ismember" "members"
      "ispc"    "getos"
      "isunix"  "getos"
      "j"       "percenti"
      "kaiser"  "window"
      "keyboard" "pause"
      "listdlg" "x_choose"
      "loglog"  "plot2d"
      "lookfor" "apropos"
      "lower"   "convstr"
      "magic"   "testmatrix"
      "matlabroot" "SCI"
      "mfilename"  "get_absolute_file_path"
      "mod"     "pmodulo"
      "more"    "lines"
      "nan"     "percentnan"
      "nanmin"  "min"
      "nanmax"  "max"
      "nargin"  "argn"
      "nargout" "argn"
      "nonzeros" "spget"
      "num2str" "string"
      "numel"   "size"
      "ode23"   "ode"
      "ode45"   "ode"
      "ode113"  "ode"
      "ode15s"  "ode"
      "odeget"  "odeoptions"
      "paddarray"   "resize_matrix"
      "patch"   "xfpolys"
      "pcg"     "conjgrad"
      "pi"      "percentpi"
      "planerot" "givens"
      "plot3"   "param3d"
      "plottools" "ged"
      "polar"   "polarplot"
      "polyarea" "polygonmetrics"
      "polyder" "derivat"
      "polyint" "integratePoly"
      "polyval" "horner"
      "printdlg" "toprint"
      "psi"     "dlgamma"
      "questdlg" "messagebox"
      "randi"   "grand"
      "randn"   "rand"
      "randperm" "grand"
      "range"   "strange"
      "rbbox"   "rubberbox"
      "realmax" "number_properties"
      "realmin" "number_properties"
      "regexprep"  "strsubst"
      "rem"     "modulo"
      "reshape" "matrix"
      "rmfield" "null"
      "rotate3d" "rotate_axes"
      "rot90"   "flipdim"
      "run"      "exec"
      "semilogx" "plot2d"
      "semilogy" "plot2d"
      "shg"      "show_window"
      "sort"     "gsort"
      "sortrows" "gsort"
      "spdiags"  "diag"
      "sprandn"  "sprand"
      "sprandsym" "sprand"
      "spy"      "PlotSparse"
      "stairs"   "plot2d2"
      "std"      "stdev"
      "stem"     "plot2d3"
      "strfind"  "strindex"
      "strjoin"  "strcat"
      "strjust"  "justify"
      "strtrim"  "stripblanks"
      "surf2patch" "genfac3d"
      "switch"   "select"
      "tempdir"  "TMPDIR"
      "texlabel" "math_rendering_features_in_graphic"
      "text"     "xstring"
      "textscan" "msprintf"
      "trapz"    "inttrap"
      "true"    "percentt"
      "tukeywin" "window"
      "uisetcolor" "uigetcolor"
      "unix"    "unix_g"
      "upper"   "convstr"
      "urlread" "getURL"
      "var"     "variance"
      "version" "getversion"
      "vertcat" "cat"
      "waitforbuttonpress" "xclick"
      "warndlg" "messagebox"
      "webread" "getURL"
      "which"   "get_function_path"
      "whos -file" "listvarinfile"
      "xlim"    "axes_properties"
      "ylim"    "axes_properties"
      "zlim"    "axes_properties"
      "zoom"    "zoom_rect"        // + unzoom
    ]
    
    ex2key = [
      "view"    "rotation_angles"  // for "g", "w", "wb"
    ]
    // REDIRECTIONS to modules, FileExchanges, or external references
    // In ScilabFirst mode AFTER search in Scilab JAR & search in modules JAR
    // as long as the left entry is not a legal Scilab entry
    ex2ex = [
      "circshift"   "cshift"
      "Firebird"    "scidb"
      "inpolygon"   "point_in_polygon"
      "MySQL"       "scidb"
      "MSSQL"       "scidb"
      "ODBC"        "scidb"
      "PostgreSQL"  "scidb"
      "print_usage" "disp_usage"
      "python"      "PIMS"
      "Sqlite"      "scidb"
      "web"         "browseURL"
      "apifun"      "£apifun_overview"          // (if Apifun installed)
      "cglab"       "£constrained_delaunay_2"   // (if CGLAB installed)
      "ipd"         "£ShowImage"                // if IPD installed)
      "metanet"     "£bandwr"                   // "uman metanet s" will be OK
      "scimax"      "£PERCENTedispflag"         // if SciMax installed
      ]
    
    // Queries matching a scilab item but with a different meaning in an external 
    //  language. There are considered only if the priority target is external package
    exin2in = [ 
      "contour"  "contour2d"
      "conv"    "convol"
      "dos"     "unix_g"
      "end"     "dollar"
      "home"    "d£tohome"      // poor page => force "d" 
      "load"    "loadmatfile"
      "null"    "kernel"
      "pause"   "halt"
      "quiver"  "champ"         // quiver implemented in plotlib
      "square"  "squarewave"
      "system"   "unix_g"
      "type"    "unix_w"
      "ver"     "getversion"
      ]

   item = item(1)
    if or(equivType==["shortcuts" "ex2in" "ex2ex" "exin2in" "ex2key"])
        execstr("e="+equivType)
        if convstr(item)==item 
            ie = find(convstr(e(:,1))==item)
        else
            ie = find(e(:,1)==item)
        end
        if ie~=[]
            item = e(ie(1),2)
            [item, options] = uman_updateQueryFromEquiv(item, options)
        end
    end
endfunction

// -------------------------------------------------------------------------

// Copyright (C)  2013 - 2016   Samuel GOUGEON
// Utility function for uman()
function table = uman_get_atoms_forge()
    // Actually, the table may also contain references of non-ATOMS packages
    // [ module_name  module_name_on_forge  website ]
    atoms = [
    "a" "accsum"            "accsum"    ""
    "a" "AdaptiveOptics"    ""          "http://sciao.sourceforge.net/#authors"
    "a" "ADS_CoLiSyS"       ""          ""
    "a" "aerospace-toolbox" "aerospace-toolbox"	""
    "a" "aerospace_blockset" "aerospace-blockset"	""
    "a" "ampl_toolbox"      ""          "http://code.google.com/p/scilab-mip"
    "a" "Analog_and_DigitalCommunication"   ""  "http://www.geocities.ws/senthilirtt/"
    "a" "Anfislab"          ""          "http://www.tritytech.com"
    "a" "ANN_Toolbox"       "ann-toolbox" ""
    "a" "apifun"            "apifun"    ""
    "a" "ardronemodule"     ""          "https://code.google.com/p/ardronetoolbox/"
    "a" "arduino"           "arduino"   ""
    "a" "arfit"             ""          "http://www.gps.caltech.edu/~tapio/arfit/"
    "a" "assert"            "assert"    ""
    "a" "BackDoor"          "BackDoor"  ""
    "a" "BTD"               ""          "http://dcb.res.in/scilab-btd/"
    "a" "casci"             "casci"     "http://philippe.castagliola.free.fr"
    "a" "CCA"               ""          "http://people.ok.ubc.ca/ylucet/cca.html"
    "a" "celestlab"         "celestlab" ""
    "a" "celestlabx"        ""          ""
    "a" "cglab"             "cgal"      ""
    "a" "CLUSTER"           ""          "http://www.xing.com/profile/Harald_Galda"
    "a" "Computer-Vision"   "cvfunctions"  ""
    "a" "condnb"            "condnb"    ""
    "a" "Conint"            "conint"    ""
    "a" "conmin"            ""          "http://code.google.com/p/scilab-mip"
    "a" "convertlatin"      "convertlatin"  ""
    "a" "coselica"          "coselica"  ""
    "a" "CMA-ES"            ""          "http://www.cfuttrup.com"
    "a" "CPGE"              "cpge"      "http://www.demosciences.fr/"
    "a" "csv_readwrite"     "csv-readwrite" ""
    "a" "Curl"              ""          ""
    "a" "CUTEr"             "cuter"     ""
    "a" "CWA"               "CWA"       ""
    "a" "dace_scilab"       "dace-scilab"   ""
    "a" "dbldbl"            "dbldbl"    ""
    "a" "dde_toolbox"       "dde-toolbox"   ""
    "a" "DD_QD"             ""          "http://www.mi.kagu.tus.ac.jp/qupat.html"
    "a" "DeCaToKi"          ""          "http://sites.google.com/site/ronaystein/"
    "a" "Dynpeak"           ""          ""
    "a" "diffcode"          "diffcode"  ""
    "a" "digimetrie_Tools"  ""          "http://www.digimetrie.com/"
    "a" "dispmat"           "dispmat"   ""
    "a" "distfun"           "distfun"   ""
    "a" "Dynpeak"           ""          ""
    "a" "edflib"            ""          "http://www.teuniz.net/edflib/"
    "a" "emd_toolbox"       ""          "http://perso.ens-lyon.fr/patrick.flandrin/emd.html"
    "a" "EMTTOOL"           ""          ""
    "a" "E4Coder"           ""          "http://www.e4coder.com/"
    "a" "FACT"              ""          ""
    "a" "favouritedirs"     ""          ""
    "a" "femtruss"          ""          "http://code.google.com/p/scilab-mip/"
    "a" "Financial"         "Financial" ""
    "a" "FireBirdToolboxforFireBirdRobot"   ""  "http://www.nex-robotics.com/"
    "a" "fitters"           ""          ""
    "a" "floatingpoint"     "floatingpoint" ""
    "a" "fmincon"           "fmincont"  ""
    "a" "FOT"               ""          "http://www.scilab.in/fossee-scilab-toolbox/optimization-toolbox"
    "a" "fsmtbx"            ""          "http://fsmtbx.origo.ethz.ch/"
    "a" "getDmatrix"        ""          "http://uolp.unifor.br/oul/pages/academico/posgraduacao/novoSite/professorPL.jsp?Professor=3820065&p_tipo_pagina=grad"
    "a" "Global_Optim_toolbox"  ""      "http://www.sc.iitb.ac.in/~bhagyesh/Scilab/GlobalOptimizationToolbox.html"
    "a" "grocer"            ""          "http://dubois.ensae.net/grocer.html"
    "a" "guibuilder"        ""          "http://www.tritytech.com/"
    "a" "guimaker"          "guimaker"  ""
    "a" "helptbx"           "helptbx"   ""
    "a" "huffcomp"          ""          ""
    "a" "hypt"              "hypt"      ""
    "a" "hyb_auto_module"   "hyb-auto-module"   ""
    "a" "HYDROGRv50"        ""          "http://lerat.julien.free.fr/CONT_FR/HYDROGR_FR.html"
    "a" "ica_toolbox"       ""          "http://research.ics.tkk.fi/ica/fastica/"
    "a" "identification"    ""          ""
    "a" "imsls"             "imsls"     ""
    "a" "indinf"            "indinf"    ""
    "a" "intprbs"           "intprbs"   ""
    "a" "iodelay"           ""          ""
    "a" "IPD"               "IPD"       "http://www.xing.com/profile/Harald_Galda/"
    "a" "ipoptlib"          "ipoptlib"  ""
    "a" "IPT2"              ""          "http://cdac.in"
    "a" "IPT3"              ""          ""
    //"a" "IWT"               ""          "http://www.cdac.in": replaced with the next
    "a" "Integer_Wavelet_Transform_ToolBox" ""  ""
    "a" "JIMS"              "JIMS"      ""
    "a" "json"              "json"      ""
    "a" "krisp"             ""          ""
    "a" "lcc_windows"       "lcc-windows"   ""
    "a" "leftwavedenoise"   ""          "http://cas.ensmp.fr/~chaplais/fr/Ondelettes/Ondelettes/DebruitageOndelettes.html"
    "a" "libsvm"            "libsvm"    ""
    "a" "linalg"            "linalg"    ""
    "a" "lolimot"           "lolimot"   "http://code.google.com/p/scilab-mip"
    "a" "lowdisc"           "lowdisc"   ""
    "a" "lsitbx"            ""          "http://lsitbx.origo.ethz.ch/"
    "a" "makematrix"        "makematrix"    ""
    "a" "maple2scilab"      "maple2scilab"  ""
    "a" "Mascaret"          ""          "http://www.openmascaret.org/"
    "a" "Mathieu"           "mathieu"   ""
    "a" "MatrixMarket"      "matmark"   ""
    "a" "Maxplus"           ""          ""
    "a" "maxplus_petrinet"  ""          ""
    "a" "Max_Plus_Algebra"  ""          "http://www.its.ac.id/personal/index.php?id=Subiono"
    "a" "MDPtoolbox"        ""          "http://www.inra.fr/mia/T/MDPtoolbox"
    "a" "MED"               "MED"       ""
    "a" "metanet"           "metanet"   ""
    "a" "mi"                ""  "http://penglab.janelia.org/proj/mRMR/"
    "a" "microdaq"          ""          "http://embedded-solutions.pl/"
    "a" "microwave"         ""          "http://www.reveyrand.fr/uW.html"
    "a" "mingw"             "MinGW-toolbox" ""
    "a" "minphase_toolbox"  ""          ""
    "a" "mkltools"          "mkltools"  ""
    "a" "moc_toolbox"       "moc-toolbox" ""
    "a" "modbus"            "modbus"    ""
    "a" "module_lycee"      "module-lycee"  ""
    "a" "montesci"          ""          "http://www.drivehq.com/web/ka_eins/montesci/"
    "a" "MPScilab"          "mpscilab"  ""
    "a" "nan"               "nan-toolbox"   ""
    "a" "NARVAL"            ""          "http://wwwfr.uni.lu/snt/"
    "a" "neuralnetwork"     ""          "http://www.tritytech.com/"
    "a" "NIDAQ"             "nidaq"     ""
    "a" "NISP"              "nisp"      ""
    "a" "nistdataset"       "nistdataset"   ""
    "a" "Nqueens"           ""          ""
    "a" "NTG"               ""          "http://wwwfr.uni.lu/interdisciplinary_centre_for_security_reliability_and_trust/people/foued_melakessou"
    "a" "number"            "number"    ""
    "a" "OFIP"              ""          ""
    "a" "opc_client"        "opc-client" ""
    "a" "OpenRTDynamics"    ""          ""
    "a" "opti"              ""          ""
    "a" "optkelley"         "optkelley" ""
    "a" "Ortpol"            "ortpol"    ""
    "a" "Overload_Toolbox"  "overload"  ""
    "a" "PIMS"              "pims"      ""
    "a" "plotlib"           "plotlib"   ""
    "a" "portaudio"         "portaudio" ""
    "a" "predictivemodule"  ""          ""
    "a" "ProActiveConnector" ""         "http://proactive.activeeon.com"
    "a" "PSO"               "pso-toolbox" ""
    "a" "PVM"               "pvm"       ""
    "a" "quapro"            "quapro"    ""
    "a" "rdataset"          "rdataset"  ""
    "a" "regtools"          "regtools"  ""
    "a" "rest_client"       ""          ""
    "a" "RLDesign"          ""          ""
    "a" "rltool"            ""          ""
    "a" "Robotics_Toolbox"  ""          ""
    "a" "scetoexe"          "scetoexe"  ""
    "a" "Scholar_Lycee_in_English"  "module-lycee"  ""
    "a" "scibench"          "scibench"  ""
    "a" "scicheckupdate"    "scicheckupdate"    ""
    "a" "scicobyla"         "scicobyla" ""
    "a" "SciCrypt"          "scicrypt"  ""
    "a" "SciCSDP"           ""          "http://code.google.com/p/scilab-mip/"
    "a" "scidb"             "scidb"     ""
    "a" "scidemo"           "scidemo"   ""
    "a" "scidoe"            "scidoe"    ""
    "a" "sciFLT"            ""          "http://www.geocities.ws/jaime_urzua/sciFLT/sciflt.html"
    "a" "SciFreeFEM"        ""          "http://code.google.com/p/scilab-mip/"
    "a" "SciGLPK"           ""          "http://code.google.com/p/scilab-mip/"
    "a" "sciGPGPU"          ""          ""
    "a" "scilab2c"          "scilab2c"  ""
    "a" "Scilab_SSE"        ""          "http://www.scilab.org"
    "a" "Scilab_XLL"        "scilab-xll"    ""
    "a" "scilib_geo"        ""          "http://scilib-geo.area7.de/"
    "a" "SciLPSolve"        ""          "http://code.google.com/p/scilab-mip/"
    "a" "scinetcdf"         "netcdf4"   ""
    "a" "scinlopt"          ""          "http://code.google.com/p/scilab-mip"
    "a" "sciopustc"         ""          "http://opus-project.fr/"
    "a" "scipad"            ""          "http://sourceforge.net/projects/scipad/"
    "a" "sci_ipopt"         "sci-ipopt" ""
    "a" "Scholar_Lycee_in_English"  ""  "http://scilab-scholar.blogspot.com"
    "a" "serial"            ""          "github.com/sengupta/Scilab-Serial"
    "a" "serialport"        ""          ""
    "a" "SIMM"              "SIMM"      ""
    "a" "simplex"           ""          "http://code.google.com/p/scilab-mip"
    "a" "SIP"               "SIP"       "http://siptoolbox.sourceforge.net"
    "a" "siseli"            ""          "http://www.askrprojects.net/software/siseli/index.html"
    "a" "SIVP"              ""          "http://sivp.sourceforge.net/"
    "a" "SMP"               "smp"       ""
    "a" "sndfile_toolbox"   "sndfile-toolbox"   ""
    "a" "socket-server"     ""          "https://github.com/ishit/scilab-network"
    "a" "socket_toolbox"    ""          "http://www.reveyrand.fr/SOCKET2.html"
    "a" "sopi"              "sopi"      "https://github.com/pivui/sopi"
    "a" "specfun"           "specfun"   ""
    "a" "toolbox_spgl1"     "toolbox-spgl1"  ""
    "a" "spilu"             "spilu"     ""
    "a" "splspc"            "splspc"    ""
    "a" "SSP_Toolbox"       ""          "www.cdackolkata.in"
    "a" "Statpack"          "Statpack"  ""
    "a" "stftb"             "stftb"     ""
    "a" "stixbox"           "stixbox"   ""
    "a" "sudoku"            "sudoku"    ""
    "a" "supportvectormachine"  ""      ""
    "a" "swt"               "swt"       ""
    "a" "texmacs"           "texmacs"   ""
    "a" "Two-Dimensional_Transformations"   ""  "www.cdackolkata.in"
    "a" "uman"              ""          "http://sgougeon.free.fr/scilab/help/uman/scilab_en_US_help/uman.html"
    "a" "uncprb"            "uncprb"    ""
    "a" "usb_toolbox"       "usb-toolbox"   ""
    "a" "visa"              ""          ""
    "a" "Wavelab"           ""          "http://www-stat.stanford.edu/~wavelab/Wavelab_850/index_wavelab850.html"
    "a" "wgserialxcosio"    ""          "http://bioe.sourceforge.net/"
    "a" "WriteXmlExcel"     "writexmlexcel" ""
    "a" "XCPL"              ""          ""
    // "a" "xls_link"          "xls-link"  ""   // out of work
    "a" "xmllab"            "xmllab"    "http://xmllab.org"
    "a" "xmltodocbook"      "xmltodocbook"  ""
    ]

    // list on 2015-05-09
    fileExchange = [
    "fe" "addblanks"     ""  "128000"
    "fe" "bezier"        ""  "293000"
    "fe" "browseURL"     ""  "453000"
    "fe" "BVPsolver"     ""  "217000"
    "fe" "circfit"       ""  "194000"
    "fe" "combine"       ""  "237000"
    "fe" "contour3d"     ""  "contour3d"
    "fe" "cshift"        ""  "161000"
    "fe" "cummax"        ""  "348000"
    "fe" "cummin"        ""  "348000"
    "fe" "dither"        ""  "292000"
    "fe" "dxf2sci"       ""  "131000"
    "fe" "ellipfit"      ""  "194000"
    "fe" "ellipsoid"     ""  "123000"
    "fe" "ex355p"        ""  "341000"
    "fe" "fct"           ""  "213000"
    "fe" "FG_ISG500"     ""  "225000"
    "fe" "gini"          ""  "306000"
    "fe" "git"           ""  "115000"
    "fe" "GnuSciPlot"    ""  "487000"
    "fe" "ifct"          ""  "214000"
    "fe" "integratePoly" ""  "228000"
    "fe" "isDocked"      ""  "360000"
    "fe" "Ishikawa"      ""  "485000"
    "fe" "LDS1200"       ""  "226000"
    "fe" "legends_mc"    ""  "274000"
    "fe" "linfit"        ""  "250000"
    "fe" "logistics_function" "" "124000"
    "fe" "lorenzc"       ""  "307000"
    "fe" "mesharea"      ""  "370000"
    "fe" "mlbs"          ""  "mlbs"
    "fe" "mm3136A"       ""  "227000"
    "fe" "m2tabular"     ""  "297000"
    "fe" "M38XR"         ""  "232000"
    "fe" "OLPpower"      ""  "223000"
    "fe" "orderfields"   ""  "349000"
    "fe" "osciOX80"      ""  "234000"
    "fe" "partfrac"      ""  "451000"
    "fe" "pdiv_inc"      ""  "449000"
    "fe" "point_in_polygon" "" "121000"
    "fe" "polyfit"       ""  "249000"
    "fe" "polygonmetrics" "" "445000"
    "fe" "polyroots"     ""  "362000"
    "fe" "rotate3d"      ""  "369000"
    "fe" "rungekutta"    ""  "151000"
    "fe" "STL"           ""  "490000"
    "fe" "strrand"       ""  "367000"
    "fe" "strrepeat"     ""  "365000"
    "fe" "struct2json"   ""  "332000"
    "fe" "thermaSnap"    "" "230000"
    "fe" "triangu3D"     ""  "291000"
    "fe" "varspace"      ""  "364000"
    "fe" "wrscilib"      ""  "wrscilib"
    "fe" "XCPL_toolbox"  ""  ""
    "fe" "zaber"         ""  "255000"
    
    "fe" "polar2D"       ""  "285000"
    "fe" "polar3D"       ""  "285000"
    "fe" "triax"         ""  "285000"
    
    "fe" "tachometer"    ""  "312000"
    "fe" "thermometer"   "" "312000"
    "fe" "variometer"    "" "312000"
    "fe" "windrose"      ""  "312000"
    
    //"fe" ""    ""  ""
    //"fe" "px2rc"       ""  "335000"
    //"fe" "strsubst"    ""  "294000"
    //"fe" "rana"        ""  "172000"
    //"fe" "thermocouples" ""  "248000"
    //"fe" "isprime"     ""  "235000"
    //"fe" "peak_detect" ""  "209000"
    //"fe" "linear"      ""  "118000"
    ]
    FEroot = "http://fileexchange.scilab.org/toolboxes/"
    fileExchange(:,4) = FEroot + fileExchange(:,4)
    
    // Forges without ATOMS
    forges = [
    "fo" "scimax"   ""  ""
    ]
    
    others = [
    "o" "GPIB"      ""  "http://www.reveyrand.fr/GPIB.html"
    "o" "int4sci"   ""  "https://github.com/Int4Sci/Int4Sci-v2"
    "o" "Labview"   ""  "http://www.scilab.org/scilab/interoperability/connection_applications/labview"
    ];
    table = [atoms ; fileExchange ; forges ; others]
endfunction
// ==================================================================

// Copyright (C)  2013 - 2016   Samuel GOUGEON
//
// This file must be used under the terms of the CeCILL.
// Its terms are available at
// http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
// 
// Utility function for uman()
function resu = uman_strsubst(text, pattern, replace, option)
    // option: "r" if the pattern is a regular expression (with delimiters and options)
    // replace: Group id: with TCL version = "\#"    with Java = "$#"
    Sclass = "java.lang.String"

    rhs = argn(2)

    if rhs>2 then
        if text==[] then
            resu = []
            return
        end
        
        sizeText = size(text)

        // Chosing the Java method: litteral, regexp, regexp once
        // ------------------------------------------------------
        if rhs>3 then
            if option=="r" then
                jmeth = 1  // "replaceAll"
            elseif or(option==["ro" "or"])
                jmeth = 3  // "replaceFirst"
            else
                msg = _("uman","%s: Wrong value for input argument #%d: Must be in the set {%s}.\n")
                error(msprintf(msg, "strsubst", 4, "''r'',''ro''"))
            end
        else
            jmeth = 2  // "replace"
            option = ""
        end
        
        // Compiling the meta-method
        // -------------------------
        try
            arrayRep = jnewInstance("arrayRep")
        catch
            jimport(Sclass)
            arrayRep = jcompile("arrayRep", ..
            ["public class arrayRep {"
             "public static String[] replaceInArray(int meth, String arr[], String[] pat, String[] rep) {"
             "    int nbelem = arr.length; "
             "    String tmp;"
             "    String[] out = new String[nbelem];"
             "    if (meth==1) { "
             "    for (int i = 0; i < nbelem; i++) { "
             "        tmp = arr[i]; "
             "        for (int p = 0; p < pat.length; p++) { "
             "            tmp = tmp.replaceAll(pat[p], rep[p]); "
             "            } "
             "        out[i] = tmp; "
             "        } "
             "    } "
             "    else if (meth==2) {"
             "    for (int i = 0; i < nbelem; i++) { "
             "        tmp = arr[i]; "
             "        for (int p = 0; p < pat.length; p++) { "
             "            tmp = tmp.replace(pat[p], rep[p]); "
             "            } "
             "        out[i] = tmp; "
             "        } "
             "    } "
             "    else {"
             "    for (int i = 0; i < nbelem; i++) { "
             "        tmp = arr[i]; "
             "        for (int p = 0; p < pat.length; p++) { "
             "            tmp = tmp.replaceFirst(pat[p], rep[p]); "
             "            } "
             "        out[i] = tmp; "
             "        } "
             "    } "
             "    return out; "
             "    }"
             "}"]);
         end
        //pattern = stripblanks(pattern)

        // Pre-processing the patterns: Java does not accept "/../modifiers" syntax.
        //  - modifiers must be inserted as (?#) switches in head of regexp
        //  - delimiters are not expected
        if option~="" then
            // http://docs.oracle.com/javase/tutorial/essential/regex/pattern.html
            // Extracting and checking delimiters:
            delims = part(pattern, 1)
            // Is at least one of opening delimiters a forbiden character?
            forbid = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
            if ~and(strcspn(delims, forbid)==length(delims)) then
                msg = _("uman","%s: Delimiters must be non alphanumeric")
                error(msprintf(msg, "strsubst"))
            end
            // Checking the closing delimiter and extracting modifiers:
            modifiers = strrchr(pattern, delims) // should include the closing delimiter
            Nunclosed = length(find((modifiers==pattern)))
            if Nunclosed>0
                msg = _("uman", "%s: the delimiter in the pattern is sometimes unclosed (%d pattern(s) concerned)")
                error(msprintf(msg, "strsubst", Nunclosed))
            end
            // Supported modifiers:
            // u => (?u) : unicode case: set by default
            // i => (?i) : case insensitive
            // m => (?m) : multiline
            // x => (?x) : ignores comments starting with # until the end of the line
            // s => (?s) : "." matches also EOL
            Modifiers = emptystr(modifiers)+"(?u"
            if or(length(modifiers)>1)
                for m = ["i" "m" "x" "s"]
                    im = grep(modifiers,m)
                    if im~=[]
                        Modifiers(im) = Modifiers(im)+m
                    end
                end
            end
            Modifiers = Modifiers + ")"
            pattern = strncpy(pattern, length(pattern)-strcspn(strrev(pattern), delims)-1)
            L = length(pattern)
            for i = 1:length(L)
                pattern(i) = Modifiers(i) + part(pattern(i), 2:L(i))
            end
        end

         // Formating argins
         // ----------------
        text    = text(:)'
        pattern = pattern(:)'
        replace = replace(:)'
        if  size(pattern,"*")==1 then
            replace = replace(1)
        elseif size(replace,"*")==1
            replace = emptystr(pattern) + replace
        elseif size(replace,"*")~=size(pattern,"*")
            msg = _("uman", "%s: Incompatible input arguments #%d and #%d: Same sizes expected.\n")
            error(msprintf(msg, "strsubst", 2, 3))
        end
        
        // Calling the routine
        // -------------------
        resu = arrayRep.replaceInArray(jmeth, text, pattern, replace)
        
        // Post-formatting the result
        resu = matrix(resu, sizeText)

    else
        resu = ""
    end
endfunction

// ==================================================================
// CONSTRAINING EXTERNAL BUGS :
//  html parsing : http://bugzilla.scilab.org/12748 (worked around) 
//  parsing "<"  : http://bugzilla.scilab.org/13877 (worked around)
//  parsing <pre>: http://bugzilla.scilab.org/show_bug.cgi?id=12588 (worked around)
//  strsubst     : http://bugzilla.scilab.org/12808  (worked around)
//  diary        : http://bugzilla.scilab.org/show_bug.cgi?id=12551 (ignored)
//  UTF8 in console: http://bugzilla.scilab.org/10748 (blocking)
// ----------------------------------------------------------------------------
//
// CHANGES 2.0 => 2.1 (2016-10-30)
// -------------------------------
// NEW FEATURES
// * New configuration variable umanAlignSyntaxes = "" | "l" | "r" (default)
//
// IMPROVEMENTS
// * help pages are now available in french.
// * Console mode: 
//   - For a better readability, syntaxes of a function
//     are now aligned. Their justifying is tunable.
//     Examples: who, unwrap, ss2tf, dae, ode, strsplit
//   - When a page has no parameters section, its description section(s) is
//     now automatically displayed, even without using the "d" option
//
// REFERENCES ADDED
// * Redirections
//   - internal: stat=>stats
//   - to Scilab: bicg, bicgstab, cgs, class, iscell, iscellstr, isinteger,
//     isjava, numel, pcg, spdiags, square => squarewave(faux-ami)
//   - to external: print_usage
// * FileExchange: GnuSciPlot, Ishikawa, STL
// * ATOMS: forge cglab=>cgal, Dynpeak, FOT, sopi, wgserialxcosio
//
// BUG FIXED
// * 'uman close u' displayed only 'close(H)', forgetting 'close'
// * Changing csvDefault('separator',..) made 'uman' failing.
// * when the <term> of a <variablelist> has an id, uman ignored the term.
// * '& ' in a <screen> area yielded an error. Ex: 'uman _atomsGettingStarted dc'
// * when a <tr> had a single <td>, the '<td>' tag could be displayed.
// * 'uman bitget lru' did not display the parameters section
// * 'uman author> wb' without section and item yielded an error.
// * 'uman figure_size wb' did not select '... Fig.figure_size=sizes ...' 
// * 'uman [ wb' missed '[a b c ..]' and other entries
// * 'uman ? wb' or 'uman # wb' yielded a Bugzilla error
// * 'uman 12 wb' was accepted while Scilab bugs are numbered starting from 77.
// * 'uman doc/>plot2d wb' did not target the 'Documentation pages' component
// * 'uman doc/><100 wb' yielded an error (no topic)
// * 'uman author> @' used '>' as part of the author name
//
// CHANGES 1.4 => 2.0 (2016-04-06)
// ------------------------------
// NEW FEATURES
// * New option "u" to display only the list of usages (types of calls)
// * disp_usage() and disp_usage(funame) implemented and documented.
// * 'uman ... wb': It is now possible to target components or/and authors
//    or/and a maximal change age, or a specific.
// * 'uman ##### wb' now targets the bug number ##### on bugzilla.
// * New full-featured option "@" to comb online archives of Scilab mailing lists.
// * Environment variables are now available (global):
//    - umanDefaultOptions, to choose your own default options (display,
//       language, etc). Command-line options are still available, to easily 
//       override default ones.
//    - umanMaxLineWidth>=50. Default = 90. Useful when the console is very wide.
//    - umanUserPaths, to point packages out of SCI\contrib and SCIHOME\contrib
//    - umanNoStyle, to cancel ASCII-conversion of bold and italic styles.
// * Partial corpus are now supported (subset of translated scilab functions
//   without the en_US version for untranslated ones)
// * Partial support to Farsi language added (still left-justified).
//
// IMPROVEMENTS
// * "uman" is now stable with Scilab 6.0 (6.0-beta1 release)
// * Tables: long lines in cells and in the history section are now wrapped 
//    ex: uman atomsSetConfig ac
// * Items that title is several-worded like "Scilab documentation format" 
//    could not be addressed neither in the console, nor directly in the help
//    browser nor in the online help, because their single-word id that is 
//    the pattern to use to address them was displayed nowhere. 
//    In "See also" sections and Summaries, their id is now displayed instead
//    of their title. Hence, 
//    uman help s  // now displays
//    * man — on line help XML file description format
//    instead of
//    * Scilab documentation format — on line help XML file description format
//    and "uman man a .." can now be used to address the related page.
// * "Screen" areas were not specifically identified and delimited.
// * The <important> tag was not supported.
// * <superscript> and <subscript> tags were not supported.
// * The list of jar files was weekly updated. It is now just set at the first
//    use of uman, and when the "r" option is used to intentionally refresh it.
// * External modules not on ATOMS but having a public forge 
//    -- such as Scimax -- are now supported.
// * For pages having multiple xml:id, all syntaxes were displayed instead of
//    only syntaxes matching the given pattern (ex: man int32)
// * It is now possible to redirect to keywords (such as graphic properties)
//    instead of to a proper item/page, usable with g or/and w options. 
//   For example, 'uman view' now suggests a redirection to 'rotation_angles'.
// * uman's URL is now displayed at loading, for comment, scoring and bug report
//
// REFERENCES ADDED
// * Redirections
//   - internal: if installed: apifun, cglab, ipd, scimax. gui
//   - to Scilab: deal, patch, surf2patch, urlread, webread
//   - to external: inpolygon, polyarea, web
// * FileExchange: strrand, rotate3d, mesharea, polygonmetrics, pdiv_inc, 
//    partfrac, browseURL
// * ATOMS: visa, scinetcdf, Computer-Vision, favouritedirs, E4Coder, FireBird,
//    BTD (bioinformatics), cglab
// * Forge (with no ATOMS): scimax
//
// BUG FIXED
// * Scilab6-ready: management +[] modified (partly?); strtok() replaced.
// * The options had become case-sensitive.
// * For a contrib module, when the name of the main directory was not strictly
//    the name of the module (as for instance with version indication), 
//    uman failed extracting pages from the jar. 
// * When a jar file had a path with space(s), uman was unable to unzip it.
// * When the japanese archive is used, wrapping of lines in summaries of 
//    untranslated sections was shortened. ex: uman call slja
// * In synopses, "&" characters were displayed as "&amp;". Ex: uman uman u
// * In examples, "&" characters were displayed as "\&". Ex: uman uman e 
// * 'uman graphics' targetted the Graphics page, instead of the summary of
//    graphical functions => 'graphics' replaced with 'plotting'.
// * 'Calling sequence' renamed 'Syntax' in Scilab 6 is taken into account.
// * 'uman Type w' did not target the right page.
// * 'uman interval w' yielded an error (while Int4Sci is installed)
// * 'uman item1|item2 w' produced an error with host()
// * 'uman $ wb' returned all reports, instead of reports intitled with some "$".
//   'uman \$ wb' had to be used instead.
// * 'uman mode wb' unexpectedly listed bugs about 'simp_mode'. 
//   As well, 'uman matrix wb' included bugs about x_matrix().
// * uman <pattern> wb: '.', '*','\', '|' special regexp characters in pattern 
//    were not escaped before calling bugzilla. patterns such that .\., ./.,
//    .*., ./, .\, ||, **, etc were unsuccessful.
// * 'uman dir|%dir_p wb' yielded many lines without 'dir' or '%dir_p' but with
//    'directory'. Parentheses embracing the alternative a|b|c.. were missing.
// * 'uman list( wb' or with any item having a "(" produced a grep error.
// * 'uman [ wb' displayed 'grep: Can not compile pattern' in infinite loop.
// * With SciMax installed, listing its summary produced an error "Invalid index"
// * When the page of an external unloaded module was targeted with "g", 
//   the message "module not loaded => not browsable" was no longer displayed.
//
// CHANGES 1.3 => 1.4 (2015-07-31)
// -------------------------------
// REFERENCES ADDED
// * Redirections
//   - internal: latex
//   - to Scilab: closereq, cputime, fgetl, fopen, fread, fwrite
// * FileExchange: varspace, strrepeat, strrand
//
// IMPROVEMENTS & BUG FIXED
// * 'uman linespec g' missed targetting 'LineSpec' in the help browser
//    case-sensitive. The 'plot' page was displayed instead.
// * After removing a contrib, 'r' did not unregister its help pages.
// * Tables: content of heading cells <th> was not upcased.
//
// CHANGES 1.2 => 1.3 (2015-07-12)
// -------------------------------
// REFERENCES ADDED
// * Redirections added: 
//    - to ATOMS: python, MySQL, MSSQL, PostgreSQL, ODBC, Sqlite, 
//        Firebird, help (x)
//    - to scilab: trapz
//    - internal: history
//    - to FileExchange: polyint
// * FileExchange: polyroot, polyIntegrate, isDocked, orderfields, 
//    varspace, cummin, cummax
// * External references added: Labview, GPIB, int4sci
//
// IMPROVEMENTS & BUGS FIXED
// * 'wb': It was not possible to look for bugs cumulated over several 
//    Scilab or unknown items separated by '|'. An error was thrown.
//    Example: uman userdata|user_data wb
// * 'uman hdf5 wb' targeted 'h5read' instead of 'hdf5' in bugzilla.
//   Now, redirections are not applied when 'wb' is used.
// * For any user's function 'MyFun', 'uman myfun' was case-sensitive
//    and did not detect MyFun.
// * Help pages: a closing tag was badly placed. Minor fixes or updates
// * Windows: uman nidaq w opened a windows directory.
// * Shortcuts for external references with 'w' dit not work.
// * uman ./. wb  and uman .\. wb  yielded a grep error
// * uman mopen a, uman overloading a : there were large vertical
//    spaces between examples (due to erroneous empty sections)
// * In a definition list, when a term included some ':', its end was 
//    trimmed from the first ':'. Example: 'uman : d'
// * External packages: the name of their main directory was case-sensitive.
//
// CHANGES 1.1 => 1.2 (2015-06-06)
// -------------------------------
// * uman did not work with Scilab 6
// * uman $ wb  yielded a java exception
// * uman gpib  failed without proposing using the "w" option.
// * Windows: unzipping help archives was not quiet in the consolebox
// * uman int16 w did not target the int8 online page instead
// * uman <pattern> wb did not display entries where <pattern> ends the line.
// * uman who wb  displayed also bugs related to "who_user"
// * added redirections: range, fzero, inpolygon, std, var, cummax, cummin,
//    circshift, histcounts, orderfields
// * uman optim : the section "Example : Passing extra parameters"  was 
//    displayed as Parameters section with title "Parameters". 
// * uman metanet s: with metanet 0.6.2-2, there was an extra \n after each line
// * uman tokens e: '=' became erroneously ' = '
// * Unprotected "<" literal characters in the optim's page made uman crashing.
//   See also http://bugzilla.scilab.org/13877
// * No ressources out of ATOMS and FileExchange were registered for "w" 
//    targeting: int4sci and GPIB added.
// * No ressources from FileExchange were registered. More than 40 selected
//    functions on FileExchange are now addressable: bezier, contour3d, cshift, 
//    ellipfit, git, struct2json, etc.
// * List of ATOMS modules updated. 
// * "r": only the page of the pattern, in the queried language, was 
//    refreshed (instead of pages of all external packages in all languages)
// * By default, uman's output was written in open diaries and paused with "j",
//   instead of being ignored unless "j" is used.
// * for an undocumented pattern, "g" was not used
// * for an undocumented pattern, "w" canceled by "s" was not used
// * uman "unknown pattern" w  called the web browser twice
// * functions grabbed on the fileExchange or other sites and inline-documented
//   could not be web-referenced and targeted by uman with "w" and "wb" options.
// * "wb": for ATOMS without forge or other contrib packages, bugzilla was targeted 
//    instead of the comment's section of the ATOMS page or the dedicated website.
// * There was no separating blank line after any code sample
// * The output width was up-bounded only by the console width. Now max 100-char
// * extra empty or blank lines at the end of a section were not removed
// * summary in ja_JP: when the short description was unbreakable and long, it
//    was displayed on the next line, leaving the first line empty.
// * "s" option was canceled by "w" (instead of canceling "w" by "s")
// * items of the "see also" section were not aligned at all
// * summaries: alignment of short items improved
// * Help pages updated

//  1.1 (2015-03-27)
//  ---
//  * Help pages updated
//  * ja_JP: Ends of sentences were not wrappable points
//  * summaries : wrapping was only partial. left justification of the unique
//     wrapped line could be wrong.
//  * "uman openserial" displayed an error
//  * "uman type" returned an error (after the implementation of the 
//     case-sensitivity management)
//  * Some error messages returned by the OS could be displayed
//  * Automatical refreshing / reloading was done once a day. Now once a week.
//  * The pattern was always case-insensitive. Now only when all in lowercase.
//  * when the pattern was finally head_commented, head_comment was displayed twice
//  * under Linux, jar files were not always properly detected & listed 
//
//  1.0  (2015-03-22)
//  ---
//  * The toolbox was not completly set
//  * There were no unitary tests (.dia.ref still to be writen)
//  * help in head_comments was not up-to-date
//  * searching the pattern was case-sensitive
//  * "L.." option was possible only as last option
//  * Itemized lists: wrapped lines were not indented
//  * Itemized lists: some lines were wrongly wrapped
//  * Examples with strings including some tages were not displayed
//  * in some pages, some %26#8212 raw entities appeared.
//  * The list of ATOMS modules (and related forge or website) was not up-to-date
//    (2015-03-20. Last = openCV Function implementation...)
//  * Summary: the path within the help was not displayed
//  * Option "a" added : All sections displayed (but bugs URLs)
//  * The title of refsections could have extra spaces.
//  * There was no help page in french
//  * "x" mode: non existing pattern did not display the 
//     "uman: "<pattern>" is not documented" message
//  * ~"wb" mode: braces "{" and "}" were not converted into their equivalent "[" "]" 
//  * "wb": symbols were not regexp-escaped before calling bugzilla search in 
//      regexp mode.
//  * "x" : priority to external patterns was dropped, since redirections are
//     called only in second. 
//  * "wb": any %# pattern prefixed with % was wrongly translated into percent#
//     before calling the online bug page 
//  * redirection: some online help or bug pages opened targetting the 
//     original pattern instead of the redirected one.
//  * "swb" opened online help and bug pages. Now "s" kills "w" and "b"
//  * "wb": when the pattern was not found, Scilab help page AND bugzilla page
//     were addressed together, instead of only Bugzilla
//  * "wb": when the pattern was not found, Scilab help page was addressed 
//     instead of the bugzilla one.
//  * redirections were called a priori, instead of only when the initial 
//    pattern was not found
//  * ATOMS modules: Many forge references were not correct. ALl checked.
//  * Tip and Caution tags were not displayed (only Note and Warning)
//  * "h" option introduced for history info
//  * "h" option (help browser) renamed into "g" (GUI). This history also updated
//  * <code> style was not reviewed and improved. 
//  * sections identification and display refactorized. For multiple sections 
//    of the same type, only the first occurence was displayed
//  * Some WARNING and NOTE were not detected
//  * In pieces of code, "#;#" style was allowed, instead of "#; #"
//  * There was a memory leak due to undeleted xml documents
//  * <code>..</code> sections could be wrapped
//  * options: when flags were used before and after the item and "L" was used,
//     most of options were ignored.
//  * Tables: missing cells yielding too short lines yielded an error.
//  * Tables: rows with only one cell were roughly processed. Now, their complex
//     content is processed as for other sectionning tags.
//  * Some <pre> sequences of programlisting were not processed correctly: their
//     id was displayed instead of the code. Example: uman fft cdLfr  in the
//     "Syntaxe longue pour la FFT mutidimensionnelle" section
//  * Mac OS: file finding implemented. Mac OS should now be supported (to be tested)
//  * German added for sections detection
//  * last implementations were no longer working under Linux. Repared
//  * xml page updated
//  * option "p" (package) renamed "x" (eXternal) 
//  * strsubst2() implemented through TCL did not support Mac OS
//    => replaced with the Java version
//  * patterns including spaces did not convert then into "_" (as in many
//     "properties" entries.
//  * Redirections : input elements (left column) were not considered whether
//     they had uppercase characters.
//  * External faux amis were not reachable. For instance, the equivalence for
//    Octave "end" was not accessible (since "end" exists in SCilab). They are
//    now reachable using the same "x" option. 
//  * "wb" on scilab functions: plot2d or plot3d were in the list for "plot"
//  * For pages presenting several functions or items, only explicitly
//     declared ones were supported with some explicit redirections.
//    They are now all supported without explicit declared redirections.
//  * The list of JAR was set each time uman was called. It is now saved, 
//     and "r" refreshes it.
//  * When redirecting, it was not possible to force some options othen than "s"
//  * external equivalences were scanned only when the primary query did not 
//     match anything. They are now scanned a priori, in order to enable
//     special processings as forcing the "s" flag for some actual queries.
//  * when the "property" or "properties" string was in the query, the display
//    of the "description" part was not forced, contrarily to what the help said.
//  * Some main categories were not easyly accessible: variables, hfd5, signal,
//     metanet, statistics, windows, xml, sparses, shortcuts, graphics, files, 
//     atoms
//  * opening default web browser under Linux was not supported
//  * "b" : if ATOMS without forge, the comments section of ATOMS page was not
//     targeted
//  * "man" renamed "uman"
//  * "x" excluded scanning Scilab pages => role modified
//  * <tbody>.. was not supported
//  * Summaries : alignements of "—" implemented
//  * Lines wrapping sometimes overflown for imbricated description lists: fixed!
//  * support to <note> and <warning> added
//  * atoms/forge data were in an external file. Now in a function
//  * <em>..</em> appeared as is in the text
//  * option "x" (eXternal packages first) implemented
//  * option "f" (force) renamed "r" (reload)
//  * help page in en_US: written and compiled
//  * language id in options could be considered as common options.
//  * "g" management improved: 
//      - loading status of contrib modules was not defined
//      - unloaded ATOMS modules were temptatively called to helpbrowser
//  * option "r" was not proposed
//  * not ATOMS modules in SCI\contrib and other external modules in 
//    SCIHOME\contrib were not detected 
//  * ATOMS modules not set for ALL USERS were not correctly processed
//  * the "s" option did not work for ATOMS modules
//  * extraction zip sous Linux
//  * toolbox : setup (to be finished)
//  * "s" was not available => added
//  * uman patconvert  yielded an error (due to strsubst2: set TXT {\} )
//  * for ATOMS modules, the web target designed for "wb" options was not correct. 
//  * "w" option used for a function of an external module was unsuccessful: 
//     there is no online help for these modules => replaced with console display
//  * Only the first GET's separators "&" in URLs  was replaced. Following ones
//     were still blocking XML parsing. Example: uman conjgrad c
//  * Description lists: indentation depth was too large. It made line-wrapping
//    mis-working. Examples:
//      uman schur cd
//      uman fft cdlfr   // largeur description incorrecte
//  * when a page describes several functions and the short description of one
//     of them was missing, an error occurred. 
//     Example: uman mprintf cd    See http://bugzilla.scilab.org/13748
//  * The diary was stopped with the option "j" even when the web was targetted
//  * "w": when the item was not found in Scilab, head_comments was called instead
//    of calling the online search page.
//  * "wb": when the item was not found in Scilab, the item was not passed to
//      the Bugzilla search engine.
